# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""The module implements the linux specific address resolution plugin."""

__author__ = "Michael Cohen <scudette@gmail.com>"
from rekall import obj
from rekall.plugins.common import address_resolver
from rekall.plugins.linux import common

class LKMModule(address_resolver.Module):
    """A Linux kernel module."""

    def __init__(self, module, **kwargs):
        self.module = module
        super(LKMModule, self).__init__(
            name=unicode(module.name),
            start=module.base,
            end=module.end,
            **kwargs)


class KernelModule(address_resolver.Module):
    """A Fake object which makes the kernel look like a module.

    This removes the need to treat kernel addresses any different from module
    addresses, and allows them to be resolved by this module.
    """

    def __init__(self, session=None, **kwargs):
        super(KernelModule, self).__init__(
            # Check if the address appears in the kernel binary.
            start=obj.Pointer.integer_to_address(
                session.profile.get_constant("_text")),
            end=session.profile.get_constant("_end"),
            name="linux",
            profile=session.profile,
            session=session, **kwargs)


class LinuxAddressResolver(address_resolver.AddressResolverMixin,
                           common.LinuxPlugin):
    """A Linux specific address resolver plugin."""

    def _EnsureInitialized(self):
        if self._initialized:
            return

        # Insert a psuedo module for the kernel
        self.AddModule(KernelModule(session=self.session))

        # Add LKMs.
        for kmod in self.session.plugins.lsmod().get_module_list():
            self.AddModule(LKMModule(kmod, session=self.session))

        self._initialized = True



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""
Darwin Process collectors.
"""

__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import plugin
from rekall import registry

from rekall.plugins import core
from rekall.plugins.darwin import common
from rekall.plugins.common import memmap


class DarwinPslist(common.ProcessFilterMixin,
                   common.AbstractDarwinTypedCommand):
    name = "pslist"

    table_header = plugin.PluginHeader(
        dict(name="Process", width=40, cname="proc", type="proc"),
        dict(name="Alive", width=8, cname="alive"),
        dict(name="PPID", cname="ppid", width=6),
        dict(name="UID", cname="uid", width=6),
        dict(name="64-bit", cname="64bit", width=6),
        dict(name="Start Time", cname="start_time", width=30, style="short"),
        dict(name="CR3", cname="cr3", width=15, style="address")
    )

    def collect(self):
        for proc in self.filter_processes():
            yield (
                proc,
                proc.obj_producers != {"dead_procs"},
                proc.p_ppid,
                proc.p_uid,
                proc.task.map.pmap.pm_task_map == "TASK_MAP_64BIT",
                proc.p_start.as_datetime(),
                proc.task.map.pmap.pm_cr3
            )


class DarwinPsxView(common.ProcessFilterMixin,
                    common.AbstractDarwinTypedCommand):
    name = "psxview"

    # pylint: disable=no-self-argument
    @registry.classproperty
    @registry.memoize
    def table_header(cls):
        header = [dict(name="Process", width=40, cname="proc", type="proc")]

        for method in cls.methods():
            header.append(dict(name=method, cname=method, width=8))

        return plugin.PluginHeader(*header)

    def collect(self):
        methods = self.methods()
        for proc in self.filter_processes():
            row = [proc]
            for method in methods:
                row.append(method in proc.obj_producers)

            yield row


class DarwinPsTree(common.AbstractDarwinTypedCommand):
    name = "pstree"

    table_header = plugin.PluginHeader(
        dict(name="Depth", cname="depth", type="DepthIndicator", width=10),
        dict(name="PID", cname="pid", width=6),
        dict(name="PPID", cname="ppid", width=6),
        dict(name="UID", cname="uid", width=6),
        dict(name="Name", cname="name", width=30)
    )

    def collect(self):
        # Get the first process from pslist.
        first_proc = self.session.plugins.search(
            "(select * from pslist where proc.pid == 0).proc").first_result
        for proc, depth in self.recurse_proc(first_proc, 0):
            yield [depth, proc.pid, proc.p_ppid, proc.p_uid, proc.command]

    def recurse_proc(self, proc, depth):
        if proc.validate():
            yield proc, depth
        for child in proc.p_children.lh_first.p_sibling:
            for subproc, subdepth in self.recurse_proc(child, depth + 1):
                yield subproc, subdepth


class DarwinMaps(common.ProcessFilterMixin, common.AbstractDarwinCommand):
    """Display the process maps."""

    __name = "maps"

    def render(self, renderer):
        renderer.table_header([
            dict(name="vm_map_entry", style="address"),
            dict(name="Proc", width=40),
            ("Start", "start", "[addrpad]"),
            ("End", "end", "[addrpad]"),
            ("Protection", "protection", "6"),
            dict(name="Map Name", wrap=False),
        ])

        for proc in self.filter_processes():
            for map in proc.task.map.hdr.walk_list(
                    "links.next", include_current=False):

                # Format the map permissions nicesly.
                protection = (
                    ("r" if map.protection.VM_PROT_READ else "-") +
                    ("w" if map.protection.VM_PROT_WRITE else "-") +
                    ("x" if map.protection.VM_PROT_EXECUTE else "-"))

                # Find the vnode this mapping is attached to.
                vnode = map.find_vnode_object()

                renderer.table_row(
                    map,
                    proc,
                    map.links.start,
                    map.links.end,
                    protection,
                    "sub_map" if map.is_sub_map else vnode.path,
                )


class DarwinVadDump(core.DirectoryDumperMixin, common.ProcessFilterMixin,
                    common.AbstractDarwinCommand):
    """Dump the VMA memory for a process."""

    __name = "vaddump"

    def render(self, renderer):
        for proc in self.filter_processes():
            if not proc.task.map.pmap:
                continue

            renderer.format("Pid: {0:6}\n", proc.p_pid)

            # Get the task and all process specific information
            task_space = proc.get_process_address_space()
            name = proc.p_comm

            for vma in proc.task.map.hdr.walk_list(
                    "links.next", include_current=False):
                filename = "{0}.{1}.{2:08x}-{3:08x}.dmp".format(
                    name, proc.p_pid, vma.links.start, vma.links.end)

                renderer.format(u"Writing {0}, pid {1} to {2}\n",
                                proc.p_comm, proc.p_pid, filename)

                with renderer.open(directory=self.dump_dir,
                                   filename=filename,
                                   mode='wb') as fd:
                    self.CopyToFile(task_space, vma.links.start,
                                    vma.links.end, fd)


class DarwinPSAUX(common.ProcessFilterMixin, common.AbstractDarwinCommand):
    """List processes with their commandline."""

    __name = "psaux"

    def render(self, renderer):
        renderer.table_header([
            ("Pid", "pid", "8"),
            ("Name", "name", "20"),
            ("Stack", "stack", "[addrpad]"),
            ("Length", "length", "8"),
            ("Argc", "argc", "8"),
            ("Arguments", "argv", "[wrap:80]")])

        for proc in self.filter_processes():
            renderer.table_row(
                proc.p_pid,
                proc.p_comm,
                proc.user_stack,
                proc.p_argslen,
                proc.p_argc,
                " ".join(proc.argv))


class DarwinMemMap(memmap.MemmapMixIn, common.ProcessFilterMixin,
                   common.AbstractDarwinCommand):
    """Prints the memory map for darwin tasks."""
    __name = "memmap"

    def _get_highest_user_address(self):
        return 0x800000000000


class DarwinMemDump(memmap.MemDumpMixin, common.ProcessFilterMixin,
                    common.AbstractDarwinCommand):
    """Dumps the memory map for darwin tasks."""


# Plugins below represent different enumeration methods for process filter:


class PsListAllProcHook(common.AbstractDarwinParameterHook):
    """List all processes by following the _allproc list head."""

    name = "allproc"

    def calculate(self):
        first = self.session.profile.get_constant_object(
            "_allproc", target="proclist").lh_first

        result = set(first.p_list)
        return [x.obj_offset for x in result]


class PsListTasksHook(common.AbstractDarwinParameterHook):
    """List all processes by following the _allproc list head."""

    name = "tasks"

    def calculate(self):
        """List processes using the processor tasks queue.

        See
        /osfmk/kern/processor.c (processor_set_things)
        """
        seen = set()

        tasks = self.session.profile.get_constant_object(
            "_tasks",
            target="queue_entry",
            vm=self.session.kernel_address_space)

        for task in tasks.list_of_type("task", "tasks"):
            proc = task.bsd_info.deref()
            if proc:
                seen.add(proc.obj_offset)

        return seen


class PsListPgrpHashHook(common.AbstractDarwinParameterHook):
    """List all processes by following the _allproc list head."""

    name = "pgrphash"

    def calculate(self):
        """Process groups are organized in a hash chain.

        xnu-1699.26.8/bsd/sys/proc_internal.h
        """
        seen = set()

        # Note that _pgrphash is initialized through:

        # xnu-1699.26.8/bsd/kern/kern_proc.c:195
        # hashinit(int elements, int type, u_long *hashmask)

        # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
        # hashinit(int elements, int type, u_long *hashmask) {
        #    ...
        # *hashmask = hashsize - 1;

        # Hence the value in _pgrphash is one less than the size of the hash
        # table.
        pgr_hash_table = self.session.profile.get_constant_object(
            "_pgrphashtbl",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    target="pgrphashhead",
                    count=self.session.profile.get_constant_object(
                        "_pgrphash", "unsigned long") + 1
                )
            )
        )

        for slot in pgr_hash_table.deref():
            for pgrp in slot.lh_first.walk_list("pg_hash.le_next"):
                for proc in pgrp.pg_members.lh_first.walk_list(
                        "p_pglist.le_next"):
                    seen.add(proc.obj_offset)

        return seen


class PsListPidHashHook(common.AbstractDarwinParameterHook):
    """List all processes by following the _allproc list head."""

    name = "pidhash"

    def calculate(self):
        """Lists processes using pid hash tables.

        xnu-1699.26.8/bsd/kern/kern_proc.c:834:
        pfind_locked(pid_t pid)
        """
        seen = set()

        # Note that _pidhash is initialized through:

        # xnu-1699.26.8/bsd/kern/kern_proc.c:194
        # pidhashtbl = hashinit(maxproc / 4, M_PROC, &pidhash);

        # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
        # hashinit(int elements, int type, u_long *hashmask) {
        #    ...
        # *hashmask = hashsize - 1;

        # Hence the value in pidhash is one less than the size of the hash
        # table.
        pid_hash_table = self.session.profile.get_constant_object(
            "_pidhashtbl",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    target="pidhashhead",
                    count=self.session.profile.get_constant_object(
                        "_pidhash", "unsigned long") + 1
                )
            )
        )

        for plist in pid_hash_table.deref():
            for proc in plist.lh_first.walk_list("p_hash.le_next"):
                if proc:
                    seen.add(proc.obj_offset)

        return seen


class DarwinPgrpHashCollector(common.AbstractDarwinCachedProducer):
    name = "pgrphash"
    type_name = "proc"


class DarwinTaskProcessCollector(common.AbstractDarwinCachedProducer):
    name = "tasks"
    type_name = "proc"


class DarwinAllProcCollector(common.AbstractDarwinCachedProducer):
    name = "allproc"
    type_name = "proc"


class DarwinPidHashProcessCollector(common.AbstractDarwinCachedProducer):
    name = "pidhash"
    type_name = "proc"



from rekall.plugins.overlays.windows import win8

win10_undocumented_amd64 = {
    # wi10.raw 18:05:45> dis "nt!MiSessionInsertImage"
    #        call 0xf8014a9d4e80                      nt!memset
    # ...    or rax, 3    <---- Base address is ORed with 3.
    #        mov dword ptr [rbp + 0x3c], 1   <--- ImageCountInThisSession
    #        mov qword ptr [rbp + 0x28], rax  <---- Address
    '_IMAGE_ENTRY_IN_SESSION': [None, {
        'Address': [0x28, ["_EX_FAST_REF"]],
        }],
    }

win10_undocumented_i386 = {
    '_IMAGE_ENTRY_IN_SESSION': [None, {
        'Address': [0x14, ["Pointer"]],
        }],
    }

win10_overlays = {
    '_MM_SESSION_SPACE': [None, {
        # Specialized iterator to produce all the _IMAGE_ENTRY_IN_SESSION
        # records. In Win10 these are stored in an AVL tree instead.
        'ImageIterator': lambda x: x.ImageTree.Root.traverse(
            type="_IMAGE_ENTRY_IN_SESSION")
    }],
}


def InitializeWindows10Profile(profile):
    """Initialize windows 10 profiles."""
    win8.InitializeWindows8Profile(profile)
    profile.add_overlay(win10_overlays)

    if profile.metadata("arch") == "AMD64":
        profile.add_overlay(win10_undocumented_amd64)
    else:
        profile.add_overlay(win10_undocumented_i386)



# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
This plugin manages the profile repository.

"""
import json
import os
import multiprocessing
import subprocess
import sys
import yaml

from rekall import io_manager
from rekall import plugin
from rekall import registry
from rekall import threadpool
from rekall import testlib
from rekall import utils


NUMBER_OF_CORES = multiprocessing.cpu_count()


class BuilderError(Exception):
    """Raised when the builder failed."""


class RepositoryManager(io_manager.DirectoryIOManager):
    """We manage the repository using YAML.

    YAML is more user friendly than JSON.
    """
    # Do not include src files in the inventory.
    EXCLUDED_PATH_PREFIX = ["src"]

    def Encoder(self, data, **options):
        if options.get("raw"):
            return utils.SmartStr(data)

        # If the user specifically wants to encode in yaml, then do so.
        if options.get("yaml"):
            return yaml.safe_dump(data, default_flow_style=False)

        return utils.PPrint(data)

    def Decoder(self, raw):
        try:
            # First try to load it with json because it is way faster.
            return super(RepositoryManager, self).Decoder(raw)
        except ValueError:
            # If that does not work, try to load it with yaml.
            return yaml.safe_load(raw)

    def _StoreData(self, name, to_write, **options):
        # The user wants to store a yaml file, we make it uncompressed.
        if options.get("yaml"):
            options["uncompressed"] = True

        return super(RepositoryManager, self)._StoreData(
            name, to_write, **options)


class RepositoryPlugin(object):
    """A plugin to manage a type of profile in the repository."""
    __metaclass__ = registry.MetaclassRegistry

    def __init__(self, session=None, **kwargs):
        """Instantiate the plugin with the provided kwargs."""
        self.args = utils.AttributeDict(kwargs)
        self.session = session
        self.pool = threadpool.ThreadPool(self.args.processes)

    def TransformProfile(self, profile):
        """Transform the profile according to the specified transforms."""
        transforms = self.args.transforms or {}
        for transform, args in transforms.items():
            if transform == "merge":
                profile["$MERGE"] = args
            else:
                raise RuntimeError("Unknown transform %s" % transform)

        return profile

    def BuildIndex(self):
        repository = self.args.repository
        for index in self.args.index:
            spec = repository.GetData(index["src"])

            built_index = self.session.plugins.build_index(
                manager=repository).build_index(spec)

            repository.StoreData(index["dest"], built_index)

    def LaunchPlugin(self, plugin_name, *pos, **kwargs):
        """Runs a plugin in another process."""
        subprocess.check_call(
            [sys.executable, plugin_name] + pos +
            ["--%s='%s'" % (k, v) for k, v in kwargs.iteritems()])

    def LaunchBuilder(self, *args):
        """Relaunch this builder with the provided parameters."""
        executable = self.args.executable
        if executable is None:
            executable = sys.argv[0]
        cmdline = [executable]

        # We are launched with the python executable.
        if "python" in executable:
            cmdline.append(sys.argv[1])

        cmdline.extend(["manage_repo", "--path_to_repository",
                        self.args.repository.location])
        cmdline.append(self.args.profile_name)
        cmdline.extend(args)

        pipe = subprocess.Popen(cmdline, stderr=subprocess.PIPE)
        _, stderr_text = pipe.communicate()
        if pipe.returncode != 0:
            error_message = stderr_text.strip().splitlines()[-1]
            raise BuilderError(error_message)

    def Build(self, renderer):
        """Implementation of the build routine."""


class WindowsGUIDProfile(RepositoryPlugin):
    """Manage a Windows profile from the symbol server."""

    def FetchPDB(self, guid, pdb_filename):
        repository = self.args.repository
        fetch_pdb = self.session.plugins.fetch_pdb()
        data = fetch_pdb.FetchPDBFile(pdb_filename=pdb_filename, guid=guid)
        repository.StoreData("src/pdb/%s.pdb" % guid, data, raw=True)

    def ParsePDB(self, guid, original_pdb_filename):
        repository = self.args.repository
        data = repository.GetData("src/pdb/%s.pdb" % guid, raw=True)
        profile_class = (self.args.profile_class or
                         original_pdb_filename.capitalize())

        with utils.TempDirectory() as temp_dir:
            pdb_filename = os.path.join(temp_dir, guid + ".pdb")
            with open(pdb_filename, "wb") as fd:
                fd.write(data)

            parse_pdb = self.session.plugins.parse_pdb(
                pdb_filename=pdb_filename,
                profile_class=profile_class)

            profile_data = json.loads(str(parse_pdb))

        profile_data = self.TransformProfile(profile_data)
        repository.StoreData("%s/%s" % (self.args.profile_name, guid),
                             profile_data)

    def ProcessPdb(self, guid, pdb_filename):
        self.session.logging.info(
            "Building profile %s/%s\n", self.args.profile_name, guid)

        # Do we need to fetch the pdb file?
        repository = self.args.repository
        if not repository.Metadata("src/pdb/%s.pdb" % guid):
            self.FetchPDB(guid, pdb_filename)

        self.ParsePDB(guid, pdb_filename)

    def Build(self, renderer, *args):
        self.guid_file = self.args.repository.GetData(self.args.guids)
        if not args:
            command = "build_all"
        else:
            command = args[0]
            args = args[1:]

        self.ParseCommand(renderer, command, args)

    def _FindPDBFilename(self, guid):
        possible_guids = self.args.possible_guid_filenames
        if not possible_guids:
            possible_guids = [self.args.profile_name + ".pdb"]

        for guid_file in possible_guids:
            try:
                self.FetchPDB(guid, guid_file)
                return guid_file
            except Exception:
                continue

        raise ValueError("Unknown pdb filename for guid %s" % guid)

    def _DecodeGUIDFromArg(self, arg):
        if "/" in arg:
            pdb_filename, guid = arg.split("/")
        else:
            guid = arg
            pdb_filename = self._FindPDBFilename(arg)

        if not pdb_filename.endswith("pdb") or len(guid) != 33:
            raise ValueError("Invalid GUID or pdb filename - e.g. "
                             "ntkrnlmp.pdb/00625D7D36754CBEBA4533BA9A0F3FE22.")

        return pdb_filename, guid

    def _AddGUIDs(self, args):
        repository = self.args.repository
        guid_file = self.args.repository.GetData(self.args.guids)
        existing_guids = dict((x, set(y)) for x, y in guid_file.iteritems())

        for arg in args:
            pdb_filename, guid = self._DecodeGUIDFromArg(arg)
            self.session.logging.info(
                "Adding GUID %s %s" % (pdb_filename, guid))
            existing_guids.setdefault(pdb_filename, set()).add(guid)

        new_guids = dict((k, sorted(v)) for k, v in existing_guids.iteritems())
        repository.StoreData(self.args.guids, new_guids, yaml=True)

    def ParseCommand(self, renderer, command, args):
        if command == "build":
            for arg in args:
                pdb_filename, guid = self._DecodeGUIDFromArg(arg)
                self.ProcessPdb(guid, pdb_filename)

        elif command == "build_all":
            self.BuildAll(renderer)

        elif command == "add_guid":
            self._AddGUIDs(args)
            self.BuildAll(renderer)

        else:
            raise RuntimeError(
                "Unknown command for %s" % self.__class__.__name__)

    def BuildAll(self, renderer):
        repository = self.args.repository
        guid_file = self.args.repository.GetData(self.args.guids)
        rejects_filename = self.args.guids + ".rejects"
        rejects = self.args.repository.GetData(rejects_filename, default={})
        reject_len = len(rejects)

        try:
            changed_files = set()
            for pdb_filename, guids in guid_file.iteritems():
                for guid in guids:
                    if guid in rejects:
                        continue

                    # If the profile exists in the repository continue.
                    if repository.Metadata(
                            "%s/%s" % (self.args.profile_name, guid)):
                        continue

                    def Reject(e, guid=guid, changed_files=changed_files):
                        print "GUID %s rejected: %s" % (guid, e)
                        rejects[guid] = str(e)
                        changed_files.remove(guid)

                    # Otherwise build it.
                    changed_files.add(guid)
                    self.pool.AddTask(
                        self.LaunchBuilder,
                        ("build", "%s/%s" % (pdb_filename, guid)),
                        on_error=Reject)

            self.pool.Stop()

            if changed_files and self.args.index or self.args.force_build_index:
                renderer.format("Building index for profile {0} from {1}\n",
                                self.args.profile_name, self.args.index)

                self.BuildIndex()

        finally:
            if len(rejects) != reject_len:
                repository.StoreData(
                    rejects_filename, utils.PPrint(rejects), raw=True)

            renderer.format("Updating inventory.\n")
            repository.StoreData("inventory", repository.RebuildInventory())


class CopyAndTransform(RepositoryPlugin):
    """A profile processor which copies and transforms."""

    def Build(self, renderer):
        repository = self.args.repository
        profile_metadata = repository.Metadata(self.args.profile_name)
        source_metadata = repository.Metadata(self.args.source)
        if not profile_metadata or (
                source_metadata["LastModified"] >
                profile_metadata["LastModified"]):
            data = repository.GetData(self.args.source)

            # Transform the data as required.
            data = self.TransformProfile(data)
            repository.StoreData(self.args.profile_name, utils.PPrint(data),
                                 raw=True)
            renderer.format("Building profile {0} from {1}\n",
                            self.args.profile_name, self.args.source)


class OSXProfile(RepositoryPlugin):
    """Build OSX Profiles."""

    def Build(self, renderer):
        repository = self.args.repository
        changed_files = False
        for source in self.args.sources:
            profile_name = "OSX/%s" % source.split("/")[-1]
            profile_metadata = repository.Metadata(profile_name)

            # Profile does not exist - rebuild it.
            if not profile_metadata:
                data = repository.GetData(source)

                # Transform the data as required.
                data = self.TransformProfile(data)
                repository.StoreData(profile_name, utils.PPrint(data),
                                     raw=True)
                renderer.format("Building profile {0} from {1}\n",
                                profile_name, source)
                changed_files = True

        if changed_files and self.args.index or self.args.force_build_index:
            renderer.format("Building index for profile {0} from {1}\n",
                            self.args.profile_name, self.args.index)

            self.BuildIndex()


class LinuxProfile(RepositoryPlugin):
    """Build Linux profiles."""

    def Build(self, renderer):
        """Linux profile location"""
        convert_profile = self.session.plugins.convert_profile(
            session=self.session,
            out_file="dummy file")  # We don't really output the profile.

        changed_files = False
        total_profiles = 0
        new_profiles = 0

        for source_profile in self.args.repository.ListFiles():
            # Find all source profiles.
            if (source_profile.startswith("src/Linux") and
                    source_profile.endswith(".zip")):

                total_profiles += 1
                profile_id = source_profile.lstrip("src/").rstrip(".zip")

                # Skip already built profiles.
                if self.args.repository.Metadata(profile_id):
                    continue

                # Convert the profile
                self.session.report_progress(
                    "Found new raw Linux profile %s. Converting...", profile_id)
                self.session.logging.info(
                    "Found new raw Linux profile %s", profile_id)

                profile_fullpath = self.args.repository.GetAbsolutePathName(
                    source_profile)
                profile = convert_profile.ConvertProfile(
                    io_manager.Factory(
                        profile_fullpath, session=self.session, mode="r"))
                if not profile:
                    self.session.logging.info(
                        "Skipped %s, Unable to convert to a Rekall profile.",
                        profile_fullpath)
                    continue

                # Add profile to the repository and the inventory
                self.args.repository.StoreData(profile_id, profile)
                new_profiles += 1
                changed_files = True

        self.session.logging.info("Found %d profiles. %d are new.",
                                  total_profiles, new_profiles)

        # Now rebuild the index
        if changed_files and self.args.index or self.args.force_build_index:
            self.BuildIndex()


class ManageRepository(plugin.Command):
    """Manages the profile repository."""

    name = "manage_repo"

    @classmethod
    def args(cls, parser):
        super(ManageRepository, cls).args(parser)
        parser.add_argument(
            "-e", "--executable",
            default=None,
            help="The path to the rekall binary. This is used for "
            "spawning multiple processes.")

        parser.add_argument(
            "--processes", default=NUMBER_OF_CORES, type="IntParser",
            help="Number of concurrent workers.")

        parser.add_argument(
            "--path_to_repository", default=".",
            help="The path to the profile repository")

        parser.add_argument(
            "--force_build_index", type="Boolean", default=False,
            help="Forces building the index.")

        parser.add_argument(
            "build_target", type="StringParser", required=False,
            help="A single target to build.")

        parser.add_argument(
            "builder_args", type="ArrayStringParser", required=False,
            help="Optional args for the builder.")

    def __init__(self, command=None, path_to_repository=None,
                 build_target=None, force_build_index=False,
                 builder_args=None, session=None, **kwargs):
        super(ManageRepository, self).__init__(session=session)
        self.command = command
        self.builder_kwargs = kwargs
        self.path_to_repository = os.path.abspath(path_to_repository)
        self.build_target = build_target
        self.force_build_index = force_build_index
        self.builder_args = builder_args or []

        # Check if we can load the repository config file.
        self.repository = RepositoryManager(
            self.path_to_repository, session=self.session)

        self.config_file = self.repository.GetData("config.yaml")

    def render(self, renderer):
        for profile_name, kwargs in self.config_file.iteritems():
            # Inject args from the commandline into the builder args.
            kwargs.update(self.builder_kwargs)
            if self.build_target and profile_name != self.build_target:
                continue

            self.session.logging.info("Building profiles for %s", profile_name)
            handler_type = kwargs.pop("type", None)
            if not handler_type:
                raise RuntimeError(
                    "Unspecified repository handler for profile %s" %
                    profile_name)

            handler_cls = RepositoryPlugin.classes.get(handler_type)
            if handler_cls is None:
                raise RuntimeError(
                    "Unknown repository handler %s" % handler_type)

            handler = handler_cls(
                session=self.session, repository=self.repository,
                profile_name=profile_name,
                force_build_index=self.force_build_index,
                **kwargs)
            handler.Build(renderer, *self.builder_args)


class TestManageRepository(testlib.DisabledTest):
    """Dont run automated tests for this tool."""



# Rekall Memory Forensics
#
# Copyright 2016 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


"""This profile is for the Mach-O file format.

References:

https://developer.apple.com/library/mac/documentation/DeveloperTools/Conceptual/MachORuntime/Reference/reference.html
http://www.opensource.apple.com/source/xnu/xnu-1456.1.26/EXTERNAL_HEADERS/Mach-O/loader.h
https://github.com/llvm-mirror/llvm/blob/release_34/include/llvm/Support/MachO.h
https://github.com/opensource-apple/xnu/blob/10.9/EXTERNAL_HEADERS/Mach-O/loader.h
"""

__author__ = ("Michael Cohen <scudette@gmail.com>",
              "Adam Sindelar <adamsh@google.com")

from rekall.plugins.overlays import basic


macho_vtypes = {
    'mach_header_64': [0x20, {
        'cputype': [None, ['Enumeration', dict(
            choices={
                1: 'VAX',
                6: 'MC680x0',
                7: 'i386',
                8: 'MIPS',
                10: 'MC98000',
                11: 'HPPA',
                12: 'ARM',
                13: 'MC88000',
                14: 'SPARC',
                15: 'i860',
                16: 'Alpha',
                18: 'PowerPC',
                (0x01000000 | 7): 'X86_64',
                (0x01000000 | 18): 'PowerPC_64',
            },
            target="unsigned int",
        )]],
        'filetype': [None, ['Enumeration', dict(
            choices={
                0x1: 'MH_OBJECT',  # relocatable object file
                0x2: "MH_EXECUTE",  # demand paged executable file
                0x3: "MH_FVMLIB",  # fixed VM shared library file
                0x4: 'MH_CORE',    # core file
                0x5: 'MH_PRELOAD',  # preloaded executable file
                0x6: 'MH_DYLIB',   # dynamicly bound shared library file
                0x7: 'MH_DYLINKER',  # dynamic link editor
                0x8: 'MH_BUNDLE',  # dynamicly bound bundle file
            },
            target="unsigned int",
        )]],
        'flags': [None, ['Flags', dict(maskmap={
            'MH_NOUNDEFS': 0x1,
            'MH_INCRLINK': 0x2,
            'MH_DYLDLINK': 0x4,
            'MH_BINDATLOAD': 0x8,
            'MH_PREBOUND': 0x10,
        })]],
        'segments': [None, ['Array', dict(
            target="segment_command_64",
            count=lambda x: x.ncmds)]],
    }],

    'segment_command_64': [0x48, {
        'cmd': [None, ['Enumeration', dict(
            choices={
                0x1: 'LC_SEGMENT',
                0x2: 'LC_SYMTAB',
                0x3: 'LC_SYMSEG',
                0x4: 'LC_THREAD',
                0x5: 'LC_UNIXTHREAD',
                0x6: 'LC_LOADFVMLIB',
                0x7: 'LC_IDFVMLIB',
                0x8: 'LC_IDENT',
                0x9: 'LC_FVMFILE',
                0xa: 'LC_PREPAGE',
                0xb: 'LC_DYSYMTAB',
                0xc: 'LC_LOAD_DYLIB',
                0xd: 'LC_ID_DYLIB',
                0xe: 'LC_LOAD_DYLINKER',
                0xf: 'LC_ID_DYLINKER',
                0x10: 'LC_PREBOUND_DYLIB',
                0x11: 'LC_ROUTINES',
                0x12: 'LC_SUB_FRAMEWORK',
                0x13: 'LC_SUB_UMBRELLA',
                0x14: 'LC_SUB_CLIENT',
                0x15: 'LC_SUB_LIBRARY',
                0x16: 'LC_TWOLEVEL_HINTS',
                0x17: 'LC_PREBIND_CKSUM',
                0x80000000 | 0x18: 'LC_LOAD_WEAK_DYLIB',
                0x19: 'LC_SEGMENT_64',
                0x1a: 'LC_ROUTINES_64',
                0x1b: 'LC_UUID',
                0x80000000 | 0x1c: 'LC_RPATH',
                0x1d: 'LC_CODE_SIGNATURE',
                0x1e: 'LC_SEGMENT_SPLIT_INFO',
                0x80000000 | 0x1f: 'LC_REEXPORT_DYLIB',
                0x20: 'LC_LAZY_LOAD_DYLIB',
                0x21: 'LC_ENCRYPTION_INFO',
                0x22: 'LC_DYLD_INFO',
                0x80000000 | 0x22: 'LC_DYLD_INFO_ONLY',
            },
            target="unsigned int")]],
        'segname': [None, ['String', dict(length=16)]],
    }],
}


class MachoProfile(basic.ProfileLP64, basic.BasicClasses):
    """A profile for Mach-O files.

    This profile contains types for both 32 and 64bit Mach-O files, although
    only the latter is actually in use by anyone (including Apple).
    """

    @classmethod
    def Initialize(cls, profile):
        super(MachoProfile, cls).Initialize(profile)
        profile.add_types(macho_vtypes)



# Rekall Memory Forensics
# Copyright (C) 2007,2008 Volatile Systems
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Original Source:
# Copyright (C) 2004,2005,2006 4tphi Research
# Author: {npetroni,awalters}@4tphi.net (Nick Petroni and AAron Walters)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
@author:       AAron Walters
@license:      GNU General Public License 2.0 or later
@contact:      awalters@volatilesystems.com
@organization: Volatile Systems

   Alias for all address spaces

"""
from rekall import registry
from rekall import utils


class Zeroer(object):
    def __init__(self):
        self.store = utils.FastStore(10, lock=True)

    def GetZeros(self, length):
        try:
            return self.store.Get(length)
        except KeyError:
            zeros = "\x00" * length
            self.store.Put(length, zeros)
            return zeros


# Keep a bunch of zeros around for speed.
ZEROER = Zeroer()


class TranslationLookasideBuffer(object):
    """An implementation of a TLB.

    This can be used by an address space to cache translations.
    """

    PAGE_SHIFT = 12
    PAGE_ALIGNMENT = (1 << PAGE_SHIFT) - 1
    PAGE_MASK = ~ PAGE_ALIGNMENT

    def __init__(self, max_size=10):
        self.page_cache = utils.FastStore(max_size)

    def Get(self, vaddr):
        """Returns the cached physical address for this virtual address."""

        # The cache only stores page aligned virtual addresses. We add the page
        # offset to the physical addresses automatically.
        result = self.page_cache.Get(vaddr & self.PAGE_MASK)

        # None is a valid cached value, it means no mapping exists.
        if result is not None:
            return result + (vaddr & self.PAGE_ALIGNMENT)

    def Put(self, vaddr, paddr):
        if vaddr & self.PAGE_ALIGNMENT:
            raise TypeError("TLB must only cache aligned virtual addresses.")

        self.page_cache.Put(vaddr, paddr)


class Run(object):
    """A container for runs."""
    __slots__ = ("start", "end", "address_space", "file_offset", "data")

    def __init__(self, start=None, end=None, address_space=None,
                 file_offset=None, data=None):
        self.start = start
        self.end = end
        self.address_space = address_space
        self.file_offset = file_offset
        self.data = data

    @utils.safe_property
    def length(self):
        return self.end - self.start

    @length.setter
    def length(self, value):
        self.end = self.start + value

    def copy(self, **kw):
        kwargs = dict(start=self.start, end=self.end,
                      address_space=self.address_space,
                      file_offset=self.file_offset,
                      data=self.data)
        kwargs.update(kw)

        return self.__class__(**kwargs)

    def __str__(self):
        if self.file_offset is None:
            return u"<%#x, %#x>" % (self.start, self.end)

        return u"<%#x, %#x> -> %#x @ %s" % (
            self.start, self.end, self.file_offset,
            self.address_space)


class BaseAddressSpace(object):
    """ This is the base class of all Address Spaces. """

    __metaclass__ = registry.MetaclassRegistry
    __abstract = True

    order = 10

    # This can be used to name the address space (e.g. process if etc).
    name = ""

    # Some useful metadata for address spaces.

    # This signifies that this address space normally operates on memory
    # images. This flag controls if this address space will participate in
    # address space autoselection for image detection. Note that it can not be
    # inherited but must be explicitly set.
    __image = False

    # This flag signifies whether this address space's contents are likely to
    # change between reads. If an address space is NOT volatile (this flag is
    # False) then reads from the same offset MUST always return the same bytes.
    volatile = False

    # This flag signifies whether this address space is for a virtual machine.
    virtualized = False

    def __init__(self, base=None, session=None, profile=None, **_):
        """Base is the AS we will be stacking on top of, opts are options which
        we may use.

        Args:
          base: A base address space to stack on top of (i.e. delegate to it for
              satisfying read requests).

          session: An optional session object.

          profile: An optional profile to use for parsing the address space
              (e.g. needed for hibernation, crash etc.)
        """
        if session is None and base is not None:
            session = base.session

        self.base = base
        if base:
            self.volatile = self.base.volatile

        self.profile = profile
        self.session = session
        if session is None:
            raise RuntimeError("Session must be provided.")

        # This is a short lived cache. If we use a static image, this cache need
        # not expire, however, when analysing a live system we need to flush the
        # cache frequently.
        self.cache = utils.AgeBasedCache(max_age=20)

    def as_assert(self, assertion, error=None):
        """Duplicate for the assert command (so that optimizations don't disable
        them)

        It had to be called as_assert, since assert is a keyword
        """
        if not assertion:
            raise ASAssertionError(
                error or "Instantiation failed for unspecified reason")

    def describe(self, addr):
        """Return a string describing an address."""
        return "%#x" % addr

    def read(self, unused_addr, length):
        """Should be overridden by derived classes."""
        if length > self.session.GetParameter("buffer_size"):
            raise IOError("Too much data to read.")

        return ZEROER.GetZeros(length)

    def get_mappings(self, start=0):
        """Generates a sequence of Run() objects.

        Each Run object describes a single range transformation from this
        address space to another address space at a potentially different
        mapped_offset.

        Runs are assumed to not overlap and are generated in increasing order.

        Args:
          start: The suggested start address we are interested in. This function
              may omit runs that lie entirely below this start address. Note:
              Runs are not adjusted to begin at the start address - it may be
              possible that this method returns a run which starts earlier than
              the specified start address.
        """
        _ = start
        return []

    def end(self):
        runs = list(self.get_mappings())
        if runs:
            last_run = runs[-1]
            return last_run.end

    def get_address_ranges(self, start=0, end=0xfffffffffffff):
        """Generates the runs which fall between start and end.

        Note that start and end are here specified in the virtual address
        space. More importantly this does not say anything about the pages in
        the physical address space - just because pages in the virtual address
        space are contiguous does not mean they are also contiguous in the
        physical address space.

        Yields:
          Run objects describing merged virtual address ranges. NOTE: These runs
          do not have file_offset or address_space members since the file_offset
          is not the same across the entire range and therefore it does not make
          sense to directly read the base address space - If you want to do
          this, use merge_base_ranges() instead.
        """
        last_voffset = last_voffset_end = 0

        for run in self.get_mappings(start=start):
            # No more runs apply.
            if run.start > end:
                break

            if run.start < start:
                # We dont care about the file_offset here since it will be
                # dropped later.
                run = run.copy(start=start)

            # This can take some time as we enumerate all the address ranges.
            self.session.report_progress(
                "%(name)s: Merging Address Ranges %(offset)#x %(spinner)s",
                offset=run.start, name=self.name)

            # Extend the last range if this range starts at the end of the last
            # one.
            if run.start == last_voffset_end:
                last_voffset_end = run.end

            else:
                # Emit the last range
                if last_voffset_end > last_voffset:
                    yield Run(start=last_voffset,
                              end=last_voffset_end)

                # Reset the contiguous range.
                last_voffset = run.start
                last_voffset_end = min(run.end, end)

        if last_voffset_end > last_voffset:
            yield Run(start=last_voffset, end=last_voffset_end)

    def merge_base_ranges(self, start=0, end=0xfffffffffffff):
        """Generates merged address ranges from get_mapping().

        This method is subtly different from get_address_ranges in that runs are
        contiguous in the base address space, hence the yielded runs have a
        valid file_offset member. Callers can safely issue read operations to
        the address space.

        Yields:
          runs which are contiguous in the base address space. This function
            is designed to produce ranges more optimized for reducing the number
            of read operations from the underlying base address space.

        """
        contiguous_voffset = 0
        contiguous_voffset_end = 0
        contiguous_poffset = 0
        last_run_length = 0
        last_as = None

        for run in self.get_mappings(start=start):
            # No more runs apply.
            if end and run.start > end:
                break

            if run.start < start:
                run = run.copy(
                    start=start,
                    file_offset=run.file_offset + start - run.start)

            # This can take some time as we enumerate all the address ranges.
            self.session.report_progress(
                "%(name)s: Merging Address Ranges %(offset)#x %(spinner)s",
                offset=run.start, name=self.name)

            # Try to join up adjacent pages as much as possible.
            if (run.start == contiguous_voffset_end and
                    run.file_offset == contiguous_poffset + last_run_length and
                    run.address_space is last_as):
                contiguous_voffset_end = min(run.end, end)
                last_run_length = contiguous_voffset_end - contiguous_voffset
                last_as = run.address_space

            else:
                if last_run_length > 0:
                    yield Run(start=contiguous_voffset,
                              end=contiguous_voffset_end,
                              address_space=last_as,
                              file_offset=contiguous_poffset)

                # Reset the contiguous range.
                contiguous_voffset = run.start
                contiguous_voffset_end = min(run.end, end)
                contiguous_poffset = run.file_offset or 0
                last_run_length = contiguous_voffset_end - contiguous_voffset
                last_as = run.address_space

        if last_run_length > 0:
            yield Run(start=contiguous_voffset,
                      end=contiguous_voffset_end,
                      address_space=last_as,
                      file_offset=contiguous_poffset)

    def is_valid_address(self, _addr):
        """Tell us if the address is valid """
        return True

    def write(self, addr, buf):
        """Write to the address space, if writable.

        The default behavior is to delegate the write to the base address space.
        If an address space has no base then this function will throw an
        IOError. Address spaces that actually implement writing should override.

        Raises:
            IOError if there is no base address space. Subclasses may raise
                under additional circumstances.

        Arguments:
            addr: The address to write at, as understood by this AS (i.e.
                a virtual address for virtual address spaces, physical for
                physical).
            buf: The data to write - most commonly a basestring instance.

        Returns:
            Number of bytes written.
        """
        if not self.base:
            raise IOError("No base address space set on %r." % self)

        return self.base.write(self.vtop(addr), buf)

    def vtop(self, addr):
        """Return the physical address of this virtual address."""
        # For physical address spaces, this is a noop.
        return addr

    def vtop_run(self, addr):
        """Returns a Run object describing where addr can be read from."""
        return Run(start=addr,
                   end=addr,
                   address_space=self,
                   file_offset=addr)

    @classmethod
    def metadata(cls, name, default=None):
        """Obtain metadata about this address space."""
        return getattr(cls, "_%s__%s" % (cls.__name__, name), default)

    def __unicode__(self):
        return self.__class__.__name__

    def __str__(self):
        return utils.SmartStr(self)

    def __repr__(self):
        return "<%s @ %#x %s>" % (
            self.__class__.__name__, hash(self), self.name)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and
                self.base == other.base)

    def get_file_address_space(self, filename):
        """Implement this to return an address space for filename."""

    def get_mapped_offset(self, filename, offset):
        """Implement this if we can map files into this address space."""

    def ConfigureSession(self, session_obj):
        """Implement this method if you need to configure the session."""

    def close(self):
        pass


# This is a specialised AS for use internally - Its used to provide
# transparent support for a string buffer so types can be
# instantiated off the buffer.
class BufferAddressSpace(BaseAddressSpace):
    __abstract = True

    @utils.safe_property
    def writable(self):
        """Buffer AS is always writable, no matter what the session says."""
        return True

    def __init__(self, base_offset=0, data='', metadata=None, **kwargs):
        super(BufferAddressSpace, self).__init__(**kwargs)
        self.fname = "Buffer"
        self.data = data
        self.base_offset = base_offset
        self._metadata = metadata or {}

    def metadata(self, key):
        return self._metadata.get(key)

    def assign_buffer(self, data, base_offset=0):
        self.base_offset = base_offset
        self.data = data

    def is_valid_address(self, addr):
        return not (addr < self.base_offset or addr > self.base_offset +
                    len(self.data))

    def read(self, addr, length):
        offset = addr - self.base_offset
        data = self.data[offset: offset + length]
        return data + ZEROER.GetZeros(length - len(data))

    def write(self, addr, data):
        if addr > len(self.data):
            raise ValueError(
                "Cannot write to offset %d of buffer with size %d.",
                addr, len(self.data))
        self.data = self.data[:addr] + data + self.data[addr + len(data):]
        return len(data)

    def get_mappings(self, start=None):
        if self.end > start:
            yield Run(start=self.base_offset,
                      end=self.end,
                      file_offset=self.base_offset,
                      address_space=self)

    def get_buffer_offset(self, offset):
        """Returns the offset in self.data for the virtual offset."""
        return offset - self.base_offset

    def __repr__(self):
        return "<%s @ %#x %s [%#X-%#X]>" % (
            self.__class__.__name__, hash(self), self.name,
            self.base_offset, self.end())

    def __len__(self):
        return len(self.data)

    def end(self):
        """Return the end address of the buffer."""
        return self.base_offset + len(self.data)


class CachingAddressSpaceMixIn(object):
    # The size of chunks we cache. This should be large enough to make file
    # reads efficient.
    CHUNK_SIZE = 32 * 1024
    CACHE_SIZE = 10

    def __init__(self, **kwargs):
        super(CachingAddressSpaceMixIn, self).__init__(**kwargs)
        self._cache = utils.FastStore(self.CACHE_SIZE)

    def read(self, addr, length):
        addr, length = int(addr), int(length)

        result = ""
        while length > 0:
            data = self.read_partial(addr, length)
            if not data:
                break

            result += data
            length -= len(data)
            addr += len(data)

        return result

    def cached_read_partial(self, addr, length):
        """Implement this to allow the caching mixin to cache these reads."""
        # By default call the next read_partial in the inheritance tree.
        return super(CachingAddressSpaceMixIn, self).read(addr, length)

    def read_partial(self, addr, length):
        if addr == None:
            return addr

        chunk_number = addr / self.CHUNK_SIZE
        chunk_offset = addr % self.CHUNK_SIZE

        # Do not cache large reads but still pad them to CHUNK_SIZE.
        if chunk_offset == 0 and length > self.CHUNK_SIZE:
            # Deliberately do a short read to avoid copying.
            to_read = length - length % self.CHUNK_SIZE
            return self.cached_read_partial(addr, to_read)

        available_length = min(length, self.CHUNK_SIZE - chunk_offset)

        try:
            data = self._cache.Get(chunk_number)
        except KeyError:
            # Just read the data from the real class.
            data = self.cached_read_partial(
                chunk_number * self.CHUNK_SIZE, self.CHUNK_SIZE)

            self._cache.Put(chunk_number, data)

        return data[chunk_offset:chunk_offset + available_length]


class PagedReader(BaseAddressSpace):
    """An address space which reads in page size.

    This automatically takes care of splitting a large read into smaller reads.
    """
    PAGE_SIZE = 0x1000
    PAGE_MASK = ~(PAGE_SIZE - 1)
    __abstract = True

    def _read_chunk(self, vaddr, length):
        """Read bytes from a virtual address.

        Args:
          vaddr: A virtual address to read from.
          length: The number of bytes to read.

        Returns:
          As many bytes as can be read within this page.
        """
        to_read = min(length, self.PAGE_SIZE - (vaddr % self.PAGE_SIZE))
        paddr = self.vtop(vaddr)
        if paddr is None:
            return ZEROER.GetZeros(to_read)

        return self.base.read(paddr, to_read)

    def _write_chunk(self, vaddr, buf):
        to_write = min(len(buf), self.PAGE_SIZE - (vaddr % self.PAGE_SIZE))
        if not to_write:
            return 0

        paddr = self.vtop(vaddr)
        if not paddr:
            return 0

        return self.base.write(paddr, buf[:to_write])

    def write(self, addr, buf):
        available = len(buf)
        written = 0

        while available > written:
            chunk_len = self._write_chunk(addr + written, buf[written:])
            if not chunk_len:
                break
            written += chunk_len

        return written

    def read(self, addr, length):
        """Read 'length' bytes from the virtual address 'vaddr'."""
        if length > self.session.GetParameter("buffer_size"):
            raise IOError("Too much data to read.")

        addr, length = int(addr), int(length)

        result = ''

        while length > 0:
            buf = self._read_chunk(addr, length)
            if not buf:
                break

            result += buf
            addr += len(buf)
            length -= len(buf)

        return result

    def is_valid_address(self, addr):
        vaddr = self.vtop(addr)
        return vaddr != None and self.base.is_valid_address(vaddr)


class RunBasedAddressSpace(PagedReader):
    """An address space which uses a list of runs to specify a mapping.

    This essentially delegates certain address ranges to other address spaces
    "mapped" into this address space.

    The runs are tuples of this form:

    (virtual_address, physical_address, length, address_space)

    - Virtual Address - An address in this address space's virtual address
      space.

    - Physical Address - An address in the delegate address space.

    - Length - The length of the mapped region.

    - Address space - the address space that should be read for this
      region. Note that the physical address above refers to addresses in this
      delegate address space.
    """

    # This is a list of (memory_offset, file_offset, length) tuples.
    runs = None
    __abstract = True

    def __init__(self, **kwargs):
        super(RunBasedAddressSpace, self).__init__(**kwargs)
        self.runs = utils.RangedCollection()

    def add_run(self, virt_addr, file_address, file_len, address_space=None,
                data=None):
        """Add a new run to this address space."""
        if address_space is None:
            address_space = self.base

        start = virt_addr  # Range start
        end = virt_addr + file_len  # Range end

        self.runs.insert(start, end,
                         Run(start=start,
                             end=end,
                             address_space=address_space,
                             file_offset=file_address,
                             data=data))

    def _read_chunk(self, addr, length):
        """Read from addr as much as possible up to a length of length."""
        start, end, run = self.runs.get_containing_range(addr)

        # addr is not in any range, pad to the next range.
        if start is None:
            end = self.runs.get_next_range_start(addr)
            if end is None:
                end = addr + length

            return ZEROER.GetZeros(min(end - addr, length))

        # Read as much as we can from this address space.
        available_length = min(end - addr, length)
        file_offset = run.file_offset + addr - start

        return run.address_space.read(file_offset, available_length)

    def _write_chunk(self, addr, buf):
        length = len(buf)
        start, end, run = self.runs.get_containing_range(addr)

        # addr is not in any range, ignore to the next range.
        if start is None:
            end = self.runs.get_next_range_start(addr)
            if end is None:
                end = addr + length

            return min(end - addr, length)

        # Write as much as we can to this run.
        available_length = min(end - addr, length)
        file_offset = run.file_offset + addr - start

        return run.address_space.write(file_offset, buf[:available_length])

    def vtop_run(self, addr):
        start, _, run = self.runs.get_containing_range(addr)
        if start is not None:
            return Run(start=addr,
                       end=run.end,
                       address_space=run.address_space,
                       file_offset=run.file_offset + addr - run.start)

    def vtop(self, addr):
        """Returns the physical address for this virtual address.

        Note that this does not mean much without also knowing the address space
        to read from. Maybe we need to change this method's prototype?
        """
        start, end, run = self.runs.get_containing_range(addr)
        if start is not None:
            if addr < end:
                return run.file_offset + addr - start

    def is_valid_address(self, addr):
        return self.vtop(addr) is not None

    def get_mappings(self, start=0):
        """Yields the mappings.

        Yields: A seqence of Run objects representing each run.
        """
        for _, _, run in self.runs:
            if start > run.end:
                continue

            yield run


class Error(Exception):
    """Address space errors."""


class ASAssertionError(Error, IOError, AssertionError):
    """The address space failed to instantiate."""


class AddrSpaceError(Error):
    """Address Space Exception.

    This exception is raised when an AS decides to not be instantiated. It is
    used in the voting algorithm.
    """

    def __init__(self):
        self.reasons = []
        Error.__init__(self, "No suitable address space mapping found")

    def append_reason(self, driver, reason):
        self.reasons.append((driver, reason))

    def __str__(self):
        result = Error.__str__(self) + "\nTried to open image as:\n"
        for k, v in self.reasons:
            result += " {0}: {1}\n".format(k, v)

        return result



#!/usr/bin/env python2

# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@google.com>"

"""A plugin to install relevant kernel modules to enable live analysis.

The intention is to allow the user to launch:

rekall live

and have Rekall install the right kernel module and connect to the driver on all
supported operating systems.
"""
import os
import subprocess
import tarfile

from rekall import plugin
from rekall import resources
from rekall import utils

from rekall.plugins.addrspaces import pmem


class Live(plugin.ProfileCommand):
    """Launch a Rekall shell for live analysis on the current system."""

    name = "live"

    PROFILE_REQUIRED = False

    @classmethod
    def args(cls, parser):
        parser.add_argument("--driver_path", default=None,
                            help="Driver file to load")

        parser.add_argument("--device", default=r"/dev/pmem",
                            help="Device name to use")

        parser.add_argument("-u", "--unload", default=None, type="Boolean",
                            help="Just unload the driver and exit.")

        parser.add_argument("-l", "--load", default=None, type="Boolean",
                            help="Just load the driver and exit.")

    def __init__(self, driver_path=None, device=r"/dev/pmem", unload=None,
                 load=None, **kw):
        super(Live, self).__init__(**kw)
        # It is OK for non privileged sessions to use the default drivers.
        if not self.session.privileged and driver_path:
            raise plugin.PluginError(
                "Installing arbitrary drivers is only available for "
                "interactive or privileged sessions.")

        self.driver_path = (driver_path or
                            resources.get_resource("MacPmem.kext.tgz"))
        if self.driver_path is None:
            raise IOError("Driver resource not found.")

        self.device = device
        self.unload = unload
        self.load = load

        # Did we load the driver? If so we need to clean up.
        self.we_started_driver = False

    def live(self):
        try:
            base_as = pmem.MacPmemAddressSpace(session=self.session,
                                               filename=self.device)
        except IOError as e:
            self.session.logging.debug("%s", e)
            tarfile_handle = tarfile.open(self.driver_path)

            # Try to extract the resource into a tempdir.
            with utils.TempDirectory() as tmp_name:
                self.session.logging.info("Unpacking driver to %s", tmp_name)
                tarfile_handle.extractall(tmp_name)

                # Change ownership of the extracted files to make sure they are
                # owned by root otherwise they will not load.
                for root, files, dirs in os.walk(tmp_name):
                    for f in files:
                        os.chown(os.path.join(root, f), 0, 0)

                    for d in dirs:
                        os.chown(os.path.join(root, d), 0, 0)

                for member_name in tarfile_handle.getnames():
                    if member_name.endswith(".kext"):
                        self.member_name = member_name.lstrip("/")
                        full_driver_path = os.path.join(tmp_name,
                                                        self.member_name)
                        self.session.logging.info(
                            "Loading driver from %s", full_driver_path)
                        res = subprocess.check_call(
                            ["kextload", full_driver_path])

                        if res != 0:
                            raise plugin.PluginError("%s. Are you root?" % e)

                        try:
                            base_as = pmem.MacPmemAddressSpace(
                                session=self.session, filename=self.device)
                            self.we_started_driver = True
                            break
                        except IOError as e:
                            self.session.logging.debug("%s", e)
                            raise plugin.PluginError("%s. Are you root?" % e)

        self.session.physical_address_space = base_as

    def close(self):
        if self.unload or self.we_started_driver:
            tarfile_handle = tarfile.open(self.driver_path)

            for member_name in tarfile_handle.getnames():
                if not member_name.endswith(".kext"):
                    continue

                self.member_name = member_name.lstrip("/")

                # Try to extract the resource into a tempdir.
                with utils.TempDirectory() as tmp_name:
                    tarfile_handle.extractall(tmp_name)
                    full_driver_path = os.path.join(tmp_name,
                                                    self.member_name)
                    self.session.logging.info(
                        "Unloading driver from %s", full_driver_path)
                    try:
                        subprocess.check_call(
                            ["kextunload",
                             os.path.join(tmp_name, self.member_name)])
                    except Exception as e:
                        # There isnt much we can do about it here.
                        self.session.logging.debug(
                            "Unable to unload driver: %s" % e)

    def render(self, renderer):
        renderer.format("Launching live memory analysis\n")
        if self.unload:
            return self.close()

        if self.load:
            # Live the driver loaded.
            return self.live()

        try:
            self.live()

            # Launch the shell.
            shell = self.session.plugins.shell()
            shell.render(renderer)
        finally:
            self.close()



from rekall import addrspace
from rekall import testlib
from rekall import session


class CustomAddressSpace(addrspace.BaseAddressSpace):

    def ConfigureSession(self, session_obj):
        session_obj.SetCache("foo", "bar", volatile=False)


class SessionTest(testlib.RekallBaseUnitTestCase):
    """Test the RunBasedAddressSpace implementation."""

    def setUp(self):
        self.session = session.Session()
        self.physical_AS = CustomAddressSpace(session=self.session)

    def testSessionCache(self):
        """Make sure session gets the correct cache."""
        with self.session:
            self.session.SetParameter("cache", "memory")

            # Set something in the session cache.
            self.session.SetCache("a", "b")

        # Make sure it is set.
        self.assertEqual(self.session.GetParameter("a"), "b")

        self.physical_AS.volatile = False
        self.session.physical_address_space = self.physical_AS

        # None volatile physical address space should use the user specified
        # cache type.
        self.assertEqual(self.session.cache.__class__.__name__, "Cache")

        # Assigning the physical address space causes the cache to be
        # purged. The cache is allowed to be purged at any time (it is only a
        # cache).
        self.assertEqual(self.session.GetParameter("a"), None)

        # Volatile physical address space forces a TimedCache
        self.physical_AS.volatile = True
        self.session.physical_address_space = self.physical_AS

        self.assertEqual(self.session.cache.__class__.__name__, "TimedCache")

        # Any parameters set by the address space should be present in the
        # session cache.
        self.assertEqual(self.session.GetParameter("foo"), "bar")



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__author__ = (
    "Michael Cohen <scudette@google.com>",
    "Adam Sindelar <adam.sindelar@gmail.com>")

from rekall import obj
from rekall import plugin
from rekall import registry

from rekall.plugins.darwin import common


class DarwinUnpListCollector(common.AbstractDarwinProducer):
    """Walks the global list of sockets in uipc_usrreq."""

    name = "unp_sockets"
    type_name = "socket"

    def collect(self):
        for head_const in ["_unp_dhead", "_unp_shead"]:
            lhead = self.session.profile.get_constant_object(
                head_const,
                target="unp_head")

            for unp in lhead.lh_first.walk_list("unp_link.le_next"):
                yield [unp.unp_socket]


class DarwinSocketsFromHandles(common.AbstractDarwinProducer):
    """Looks up handles that point to a socket and collects the socket."""

    name = "open_sockets"
    type_name = "socket"

    def collect(self):
        for fileproc in self.session.plugins.collect("fileproc"):
            if fileproc.fg_type == "DTYPE_SOCKET":
                yield [fileproc.autocast_fg_data()]


class DarwinNetstat(common.AbstractDarwinTypedCommand):
    """Prints all open sockets we know about, from any source.

    Netstat will display even connections that lsof doesn't know about, because
    they were either recovered from an allocation zone, or found through a
    secondary mechanism (like system call handler cache).

    On the other hand, netstat doesn't know the file descriptor or, really, the
    process that owns the connection (although it does know the PID of the last
    process to access the socket.)

    Netstat will also tell you, in the style of psxview, if a socket was only
    found using some of the methods available.
    """

    name = "netstat"

    @classmethod
    def methods(cls):
        """Return the names of available socket enumeration methods."""
        # Find all the producers that collect procs and inherit from
        # AbstractDarwinCachedProducer.
        methods = []
        for subclass in common.AbstractDarwinProducer.classes.itervalues():
            # We look for a plugin which is a producer and a darwin command.
            if (issubclass(subclass, common.AbstractDarwinCommand) and
                    issubclass(subclass, plugin.Producer) and
                    subclass.type_name == "socket"):
                methods.append(subclass.name)
        methods.sort()

        return methods

    @registry.classproperty
    @registry.memoize
    def table_header(cls):  # pylint: disable=no-self-argument
        header = [dict(name="Socket", cname="socket", type="socket", width=60)]
        for method in cls.methods():
            header.append(dict(name=method, cname=method, width=12))

        return plugin.PluginHeader(*header)

    def collect(self):
        methods = self.methods()

        for socket in sorted(self.session.plugins.collect("socket"),
                             key=lambda socket: socket.last_pid):
            row = [socket]
            for method in methods:
                row.append(method in socket.obj_producers)

            yield row


class DarwinGetArpListHead(common.AbstractDarwinParameterHook):
    """

    One version of arp_init looks like this:

    void
    arp_init(void)
    {
        VERIFY(!arpinit_done);

        LIST_INIT(&llinfo_arp); // <-- This is the global we want.

        llinfo_arp_zone = zinit(sizeof (struct llinfo_arp),
            LLINFO_ARP_ZONE_MAX * sizeof (struct llinfo_arp), 0,
            LLINFO_ARP_ZONE_NAME);
        if (llinfo_arp_zone == NULL)
            panic("%s: failed allocating llinfo_arp_zone", __func__);

        zone_change(llinfo_arp_zone, Z_EXPAND, TRUE);
        zone_change(llinfo_arp_zone, Z_CALLERACCT, FALSE);

        arpinit_done = 1;
    }

    Disassembled, the first few instructions look like this:
     0x0 55                   PUSH RBP
     0x1 4889e5               MOV RBP, RSP
     0x4 803d65e9400001       CMP BYTE [RIP+0x40e965], 0x1
     0xb 7518                 JNZ 0xff80090a7f95
     0xd 488d3dee802900       LEA RDI, [RIP+0x2980ee]
    0x14 488d35f5802900       LEA RSI, [RIP+0x2980f5]
    0x1b baf3000000           MOV EDX, 0xf3

    # This is a call to kernel!panic (later kernel!assfail):
    0x20 e80b6c1400           CALL 0xff80091eeba0

    # This is where it starts initializing the linked list:
    0x25 48c70548e94000000000 MOV QWORD [RIP+0x40e948], 0x0
         00
    0x30 488d0d0e812900       LEA RCX, [RIP+0x29810e]
    """
    name = "disassembled_llinfo_arp"

    PANIC_FUNCTIONS = (u"__kernel__!_panic", u"__kernel__!_assfail")

    def calculate(self):
        resolver = self.session.address_resolver
        arp_init = resolver.get_constant_object("__kernel__!_arp_init",
                                                target="Function")
        instructions = iter(arp_init.Decompose(20))

        # Walk down to the CALL mnemonic and use the address resolver to
        # see if it calls one of the panic functions.
        for instruction in instructions:
            # Keep spinning until we get to the first CALL.
            if instruction.mnemonic != "CALL":
                continue

            # This is absolute:
            target = instruction.operands[0].value
            _, names = resolver.get_nearest_constant_by_address(target)
            if not names:
                return obj.NoneObject("Could not find CALL in arp_init.")

            if names[0] not in self.PANIC_FUNCTIONS:
                return obj.NoneObject(
                    "CALL was to %r, which is not on the PANIC list."
                    % names)

            # We verified it's the right CALL. MOV should be right after it,
            # so let's just grab it.
            mov_instruction = next(instructions)
            if mov_instruction.mnemonic != "MOV":
                return obj.NoneObject("arp_init code changed.")

            offset = (mov_instruction.operands[0].disp
                      + mov_instruction.address
                      + mov_instruction.size)
            address = self.session.profile.Object(type_name="address",
                                                  offset=offset)

            llinfo_arp = self.session.profile.Object(
                type_name="llinfo_arp",
                offset=address.v())

            if llinfo_arp.isvalid:
                return llinfo_arp.obj_offset

            return obj.NoneObject("llinfo_arp didn't validate.")


class DarwinArp(common.AbstractDarwinProducer):
    """Show information about arp tables."""

    name = "arp"
    type_name = "rtentry"

    def collect(self):
        llinfo_arp = self.session.address_resolver.get_constant_object(
            "__kernel__!_llinfo_arp",
            target="Pointer",
            target_args=dict(target="llinfo_arp"))

        if not llinfo_arp:
            # Must not have it in the profile. Try asking the session hook
            # for the address.
            offset = self.session.GetParameter("disassembled_llinfo_arp")
            if not offset:
                self.session.logging.error(
                    "Could not find the address of llinfo_arp.")
                return

            llinfo_arp = self.session.profile.Object(
                type_name="llinfo_arp", offset=offset)

        for arp_hit in llinfo_arp.walk_list("la_le.le_next"):
            yield [arp_hit.la_rt]


class DarwinRoute(common.AbstractDarwinCommand):
    """Show routing table."""

    __name = "route"

    RNF_ROOT = 2

    def rn_walk_tree(self, h):
        """Walks the radix tree starting from the header h.

        This function is taken from
        xnu-2422.1.72/bsd/net/radix.c: rn_walk_tree()

        Which is why it does not conform to the style guide.

        Note too that the darwin source code abuses C macros:

        #define rn_dupedkey     rn_u.rn_leaf.rn_Dupedkey
        #define rn_key          rn_u.rn_leaf.rn_Key
        #define rn_mask         rn_u.rn_leaf.rn_Mask
        #define rn_offset       rn_u.rn_node.rn_Off
        #define rn_left         rn_u.rn_node.rn_L
        #define rn_right        rn_u.rn_node.rn_R

        And then the original code does:
        rn = rn.rn_left

        So we replace these below.
        """
        rn = h.rnh_treetop

        seen = set()

        # First time through node, go left */
        while rn.rn_bit >= 0:
            rn = rn.rn_u.rn_node.rn_L

        while rn and rn not in seen:
            base = rn

            seen.add(rn)

            # If at right child go back up, otherwise, go right
            while (rn.rn_parent.rn_u.rn_node.rn_R == rn and
                   not rn.rn_flags & self.RNF_ROOT):
                rn = rn.rn_parent

            # Find the next *leaf* to start from
            rn = rn.rn_parent.rn_u.rn_node.rn_R
            while rn.rn_bit >= 0:
                rn = rn.rn_u.rn_node.rn_L

            next = rn

            # Process leaves
            while True:
                rn = base
                if not rn:
                    break

                base = rn.rn_u.rn_leaf.rn_Dupedkey
                if not rn.rn_flags & self.RNF_ROOT:
                    yield rn

            rn = next
            if rn.rn_flags & self.RNF_ROOT:
                return

    def render(self, renderer):
        renderer.table_header(
            [("Source IP", "source", "20"),
             ("Dest IP", "dest", "20"),
             ("Interface", "interface", "9"),
             ("Sent", "sent", "8"),
             ("Recv", "recv", "8"),
             ("Time", "timestamp", "24"),
             ("Expires", "expires", "8"),
             ("Delta", "delta", "8")])

        route_tables = self.profile.get_constant_object(
            "_rt_tables",
            target="Array",
            target_args=dict(
                count=32,
                target="Pointer",
                target_args=dict(
                    target="radix_node_head")))

        for node in self.rn_walk_tree(route_tables[2]):
            rentry = node.dereference_as("rtentry")

            renderer.table_row(
                rentry.source_ip,
                rentry.dest_ip,
                rentry.name,
                rentry.sent, rentry.rx,
                rentry.base_calendartime,
                rentry.rt_expire,
                rentry.delta)


class DarwinIfnetHook(common.AbstractDarwinParameterHook):
    """Walks the global list of interfaces.

    The head of the list of network interfaces is a kernel global [1].
    The struct we use [2] is just the public part of the data [3]. Addresses
    are related to an interface in a N:1 relationship [4]. AF-specific data
    is a normal sockaddr struct.

    References:
      1:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/dlil.c#L254
      2:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/if_var.h#L528
      3:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/dlil.c#L188
      4:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/if_var.h#L816
    """

    name = "ifconfig"

    # ifnet_head is the actual extern holding ifnets and seems to be an
    # improvement over dlil_ifnet_head, which is a static and used only in the
    # dlil (stands for data link interface, I think?) module.
    IFNET_HEAD_NAME = ("_ifnet_head", "_dlil_ifnet_head")

    def calculate(self):
        ifnet_head = obj.NoneObject("No ifnet global names given.")
        for name in self.IFNET_HEAD_NAME:
            ifnet_head = self.session.profile.get_constant_object(
                name,
                target="Pointer",
                target_args=dict(
                    target="ifnet"))

            if ifnet_head:
                break

        return [x.obj_offset for x in ifnet_head.walk_list("if_link.tqe_next")]


class DarwinIfnetCollector(common.AbstractDarwinCachedProducer):
    name = "ifconfig"
    type_name = "ifnet"


class DarwinIPFilters(common.AbstractDarwinCommand):
    """Check IP Filters for hooks."""

    __name = "ip_filters"

    def render(self, renderer):
        renderer.table_header([
            ("Context", "context", "10"),
            ("Filter", "filter", "16"),
            ("Handler", "handler", "[addrpad]"),
            ("Symbol", "symbol", "20")])

        resolver = self.session.address_resolver
        for list_name in ["_ipv4_filters", "_ipv6_filters"]:
            filter_list = self.profile.get_constant_object(
                list_name, target="ipfilter_list")

            for item in filter_list.tqh_first.walk_list("ipf_link.tqe_next"):
                filter = item.ipf_filter
                name = filter.name.deref()
                handler = filter.ipf_input.deref()
                renderer.table_row("INPUT", name, handler,
                                   resolver.format_address(handler))

                handler = filter.ipf_output.deref()
                renderer.table_row("OUTPUT", name, handler,
                                   resolver.format_address(handler))

                handler = filter.ipf_detach.deref()
                renderer.table_row("DETACH", name, handler,
                                   resolver.format_address(handler))



# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""The module implements an OSX specific address resolution plugin."""

__author__ = "Michael Cohen <scudette@gmail.com>"
import re

from rekall.plugins.common import address_resolver
from rekall.plugins.darwin import common


class KModModule(address_resolver.Module):
    """A darwin kernel module."""

    def __init__(self, kmod, **kwargs):
        self.kmod = kmod
        start = kmod.address.v()
        super(KModModule, self).__init__(
            name=unicode(kmod.name),
            start=start,
            end=start + kmod.size.v(),
            **kwargs)

        # We currently only support the kernel's profile. In future we should
        # write a Mach-O parser to extract symbols from binaries.
        if self.name == "__kernel__":
            self.profile = self.session.profile


class DarwinAddressResolver(address_resolver.AddressResolverMixin,
                            common.AbstractDarwinCommand):
    """A Darwin specific address resolver plugin."""

    def _EnsureInitialized(self):
        if self._initialized:
            return

        # Add kernel modules.
        for kmod in self.session.plugins.lsmod().get_module_list():
            self.AddModule(KModModule(kmod, session=self.session))

        self._initialized = True



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__author__ = "Michael Cohen <scudette@google.com>"

from rekall import plugin
from rekall.plugins.darwin import common


class DarwinHandles(common.ProcessFilterMixin, common.AbstractDarwinProducer):
    """Walks open files of each proc and collects the fileproc.

    This is the same algorithm as lsof, but aimed at just collecting the
    fileprocs, without doing anything with them, or sorting.
    """

    name = "handles"
    type_name = "fileproc"

    def collect(self):
        for proc in self.filter_processes():
            for _, fileproc, _ in proc.get_open_files():
                yield [fileproc]


class DarwinLsof(common.AbstractDarwinTypedCommand):
    """Walks open files of each proc in order and prints PID, FD and the handle.

    Each process has an array of pointers to fileproc structs - the offset into
    the array is the file descriptor and each fileproc struct represents a
    handle on some resource. A type field in the fileproc determines the type
    of the resource pointed to from the fileproc (e.g. vnode, socket, pipe...).
    """

    name = "lsof"

    table_header = plugin.PluginHeader(
        dict(name="Process", cname="proc", type="proc",
             columns=[
                 dict(name="Command", cname="command", width=16),
                 dict(name="PID", cname="pid", width=8),
                 dict(name="UID", cname="p_uid", width=8)
             ]),
        dict(name="FD", cname="fd", width=5),
        dict(name="Handle", cname="fileproc", type="fileproc")
    )

    def collect(self):
        procs = self.session.plugins.collect("proc").collect()

        for proc in sorted(procs, key=lambda proc: proc.pid):
            for fd, fileproc, _ in proc.get_open_files():
                yield (proc, fd, fileproc)



# Rekall Memory Forensics
# Copyright (c) 2012, Michael Cohen <scudette@gmail.com>
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#


"""A Rekall Memory Forensics scanner which uses yara."""
import yara

from rekall import scan
from rekall import testlib
from rekall import plugin
from rekall import utils


class BaseYaraASScanner(scan.BaseScanner):
    """An address space scanner for Yara signatures."""
    overlap = 1024

    def __init__(self, rules=None, **kwargs):
        super(BaseYaraASScanner, self).__init__(**kwargs)
        self.rules = rules
        self.hits = []
        self.base_offset = None

    def _match_rules(self, buffer_as):
        """Compatibility for yara modules.

        Unfortunately there are two different implementations of the yara python
        bindings:

        # The original upstream source.
        http://plusvic.github.io/yara/

        # The version which is installed using pip install.
        https://github.com/mjdorma/yara-ctypes

        These do not work the same and so we need to support both.

        Yields:
          a tuple of (offset, rule_name, name, value)
        """
        matches = self.rules.match(data=buffer_as.data)
        # yara-cpython bindings from pip.
        if type(matches) is dict:
            for _, matches in matches.items():
                for match in matches:
                    for string in match["strings"]:
                        hit_offset = string["offset"] + buffer_as.base_offset

                        yield (match["rule"], hit_offset,
                               string["identifier"], string["data"])

        else:
            # native bindings from http://plusvic.github.io/yara/
            for match in matches:
                for buffer_offset, name, value in match.strings:
                    hit_offset = buffer_offset + buffer_as.base_offset
                    yield (match.rule, hit_offset, name, value)

    def check_addr(self, scan_offset, buffer_as=None):
        # The buffer was changed - we scan the entire buffer and record the
        # hits - then we can feed it to the Rekall scan framework.
        if self.base_offset != buffer_as.base_offset:
            self.base_offset = buffer_as.base_offset
            self.hits = []

            for rule, offset, name, value in self._match_rules(buffer_as):
                self.hits.append((rule, offset, name, value))

        if self.hits and scan_offset == self.hits[0][1]:
            return self.hits.pop(0)

    def skip(self, buffer_as, offset):
        # Skip the entire buffer.
        if not self.hits:
            return len(buffer_as.data)

        next_hit = self.hits[0][1]
        return next_hit - offset


class YaraScanMixin(object):
    """A common implementation of yara scanner.

    This should be mixed with the process filter.
    """

    name = "yarascan"

    @classmethod
    def args(cls, parser):
        super(YaraScanMixin, cls).args(parser)

        parser.add_argument("--hits", default=1E8, type="IntParser",
                            help="Quit after finding this many hits.")


        parser.add_argument("--string", default=None,
                            help="A verbatim string to search for.")

        parser.add_argument("--binary_string", default=None,
                            help="A binary string (encoded as hex) to search "
                            "for. e.g. 000102[1-200]0506")

        parser.add_argument("--yara_file", default=None,
                            help="The yara signature file to read.")

        parser.add_argument("--yara_expression", default=None,
                            help="If provided we scan for this yarra "
                            "expression.")

        parser.add_argument(
            "--scan_physical", default=False, type="Boolean",
            help="If specified we scan the physcial address space. Note that "
            "by default we scan the address space of the specified processes "
            "(or if no process selectors are specified, the kernel).")

    def __init__(self, string=None, scan_physical=False,
                 yara_file=None, yara_expression=None, binary_string=None, hits=10,
                 **kwargs):
        """Scan using yara signatures."""
        super(YaraScanMixin, self).__init__(**kwargs)
        self.hits = hits
        if yara_expression:
            self.rules_source = yara_expression
            self.rules = yara.compile(source=self.rules_source)

        elif binary_string:
            self.compile_rule(
                'rule r1 {strings: $a = {%s} condition: $a}' % binary_string
                )
        elif string:
            self.compile_rule(
                'rule r1 {strings: $a = "%s" condition: $a}' % string
                )

        elif yara_file:
            self.compile_rule(open(yara_file).read())
        else:
            raise plugin.PluginError("You must specify a yara rule file or "
                                     "string to match.")

        self.scan_physical = scan_physical

    def compile_rule(self, rule):
        self.rules_source = rule
        try:
            self.rules = yara.compile(source=rule)
        except Exception as e:
            raise plugin.PluginError(
                "Failed to compile yara expression: %s" % e)

    def generate_hits(self, address_space, end=None):
        count = 0
        scanner = BaseYaraASScanner(
            profile=self.profile, session=self.session,
            address_space=address_space,
            rules=self.rules)

        for hit in scanner.scan(maxlen=end):
            yield hit

            count += 1
            if count >= self.hits:
                break

    def render_scan_physical(self, renderer):
        """This method scans the physical memory."""
        for rule, address, _, _ in self.generate_hits(
                self.physical_address_space):
            renderer.format("Rule: {0}\n", rule)

            context = self.physical_address_space.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render_kernel_scan(self, renderer):
        for rule, address, _, _ in self.generate_hits(
                self.kernel_address_space):
            renderer.format("Rule: {0}\n", rule)
            owner = self.session.address_resolver.format_address(address)
            if not owner:
                owner = "Unknown"

            renderer.format("Owner: {0}\n", owner)

            context = self.kernel_address_space.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render_task_scan(self, renderer, task):
        """Scan a task's address space."""
        end = self.session.GetParameter("highest_usermode_address")
        task_as = task.get_process_address_space()

        for rule, address, _, _ in self.generate_hits(task_as, end=end):
            renderer.format("Rule: {0}\n", rule)

            renderer.format(
                "Owner: {0} ({1})\n", task,
                self.session.address_resolver.format_address(address))

            context = task_as.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render(self, renderer):
        """Render output."""
        cc = self.session.plugins.cc()
        with cc:
            if self.scan_physical:
                return self.render_scan_physical(renderer)

            elif self.filtering_requested:
                for task in self.filter_processes():
                    cc.SwitchProcessContext(task)

                    self.render_task_scan(renderer, task)

            # We are searching the kernel address space
            else:
                return self.render_kernel_scan(renderer)


class TestYara(testlib.SimpleTestCase):
    """Test the yarascan module."""

    PARAMETERS = dict(commandline="yarascan --string %(string)s --hits 10")



from rekall import addrspace
from rekall import testlib
from rekall.plugins.windows.malware import apihooks


class TestHookHeuristics(testlib.RekallBaseUnitTestCase):
    """Test the hook detection heuristic.

    The actual test cases are generated using the nasm assembler in:

    rekall/src/hooks/amd64.asm and rekall/src/hooks/i386.asm
    """

    def testHook(self):
        session = self.MakeUserSession()

        # The target address should be fixed at this offset.
        target = 0x100

        heuristic = apihooks.HookHeuristic(session=session)

        profile = session.profile = session.LoadProfile("tests/hooks")
        for arch in ["AMD64", "I386"]:
            for test_case in profile.data[arch]:
                offset = test_case["offset"]
                # Test case data is the assembly snippet mapped at the specified
                # offset in the address space.
                address_space = addrspace.BufferAddressSpace(
                    data=test_case["data"].decode("base64"),
                    session=session, base_offset=offset)

                function = session.profile.Function(
                    offset=offset, vm=address_space, name=test_case["name"],
                    mode=arch)

                # Detect the jump in this function
                destination = heuristic.Inspect(function)

                # All hooks in test cases go to the same target offset (0x100).
                self.assertEqual(destination, target)



"""Tests for profile_index."""

import logging
import unittest

from rekall import session
from rekall import testlib
from rekall.plugins.common import profile_index


class SymbolOffsetIndexTest(testlib.RekallBaseUnitTestCase):
    def setUp(self):

        self.test_index_data = {
            "$METADATA":
            {
                "ProfileClass": "SymbolOffsetIndex",
                "Type": "SymbolOffsetIndex",
                "Version": 1,
                "BaseSymbol": "c",
            },
            "$INDEX":
            {
                "$PROFILES":
                {
                    "P1": { "LastModified": 12345 },
                    "P1-1": { "LastModified": 12346 },
                    "P3": { "LastModified": 12347 },
                },
                "$TRAITS":
                {
                    "P1":
                    [
                        [["a", -2]],
                    ],
                    "P1-1":
                    [
                        [["a", -3]],
                    ],
                    "P3":
                    [
                        [["d", 1]],
                        [["e", 4]]
                    ],
                },
                "$HASHES":
                {
                }
            }
        }

        self.profiles = [
            ("P1", {
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 3
                }
            }),
            ("P1-DUPLICATE", {  # This is a duplicate profile
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 3,
                }
            }),
            ("P1-1", {  # P1-1 simulates a slightly newer P1 profile
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 4
                }
            }),
            ("P3", {  # P3 simulated a completely different profile
                "$CONSTANTS":
                {
                    "b": 3,
                    "c": 5,
                    "d": 6,
                    "e": 9
                }
            }),
        ]

        self.dummy_index = profile_index.SymbolOffsetIndex.LoadProfileFromData(
            self.test_index_data, session=session.Session())

    def testHashingIsStable(self):
        """Test that hashing the same profile twice leads to the same hash."""
        hash1 = profile_index.SymbolOffsetIndex.CalculateRawProfileHash(
            self.profiles[0][1])
        hash2 = profile_index.SymbolOffsetIndex.CalculateRawProfileHash(
            self.profiles[0][1])
        self.assertEqual(hash1, hash2)

    def testLookupProfileWorksOnProfilesInTheIndex(self):
        # This emulates having parsed kallsyms
        # We fake-find a "P1" profile on a live machine
        symbols = self.profiles[0][1].get("$CONSTANTS")

        profiles = self.dummy_index.LookupProfile(symbols)
        self.assertEqual(len(profiles), 1)  # Only 1 profile matches
        self.assertEqual(profiles[0][0], "P1")  # It's "P1"
        self.assertEqual(profiles[0][1], 1)  # And only 1 trait matched

    def testLookupProfileWorksWithKaslr(self):
        # We're gonna SHIFT P1 by 0x20000, just like the Linux kernel does
        profile = self.profiles[0][1]
        symbols = dict([(i[0], i[1]+0x200000)
                        for i in profile["$CONSTANTS"].iteritems()])


        profiles = self.dummy_index.LookupProfile(symbols)
        self.assertEqual(len(profiles), 1)  # Only 1 profile matches
        self.assertEqual(profiles[0][0], "P1")  # It's "P1"
        self.assertEqual(profiles[0][1], 1)  # And only 1 trait matched

    def testLookupProfileDetectsUnknownProfiles(self):
        # We'll have at least 3 cases where profile matching will find new
        # profiles:
        #   1) No match. If no profile matches, this is clearly a new profile.
        #   2) Partial match. A profile that only matches some traits is a new
        #   profile that clashes with a known profile in the repository.
        #   3) Several matches. A profile that matches more than one profile in
        #   the index is a new profile that clashes with several profiles and
        #   affects the quality of the index.
        #
        # Additionally, there's a chance a new profile may remain undiscovered
        # when it matches all traits of a currently known profile, yet is
        # actually slightly different.

        # Unknown profile.
        symbols1 = { "x": 99, "c": 14}
        profiles = self.dummy_index.LookupProfile(symbols1)
        self.assertEqual(len(profiles), 0)

        # Partial match
        symbols2 = { "c": 5, "d": 6, "e": 20}
        profiles = self.dummy_index.LookupProfile(symbols2)
        self.assertEqual(len(profiles), 1)

        # Only 1 out of 2 traits matches from P3
        profile = profiles[0][0]
        num_matched_traits = profiles[0][1]
        total_traits = len(self.dummy_index.traits[profile])
        self.assertEqual(num_matched_traits, 1)
        self.assertEqual(total_traits, 2)

        # Several profile matches.
        #   a is at -2 from c (matching P1's trait)
        #   d is at +3 from c (matching one of P3's traits)
        symbols3 = { "a": 3, "c": 5, "d": 6 }
        profiles = self.dummy_index.LookupProfile(symbols3)
        # More than 1 profile matches will mean this profile is new and that we
        # need to recompute the index.
        self.assertEqual(len(profiles), 2)
        self.assertListEqual(sorted([p[0] for p in profiles]),
                             ["P1", "P3"])


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# Rekall Memory Forensics
#
# Based on the source code from
# Volatools Basic
# Copyright (C) 2007 Komoku, Inc.
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com>
# Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
# Mike Auty <mike.auty@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# The source code in this file was inspired by the excellent work of
# Brendan Dolan-Gavitt. Background information can be found in
# the following reference:
# "The VAD Tree: A Process-Eye View of Physical Memory," Brendan Dolan-Gavitt

# pylint: disable=protected-access

import re

from rekall import scan
from rekall import utils
from rekall.plugins import core
from rekall.plugins.addrspaces import intel
from rekall.plugins.common import pfn
from rekall.plugins.windows import address_resolver
from rekall.plugins.windows import common
from rekall.plugins.windows import pagefile


class VADInfo(common.WinProcessFilter):
    """Dump the VAD info"""

    __name = "vadinfo"

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.format("Pid: {0:6}\n", task.UniqueProcessId)

            count = 0
            for count, vad in enumerate(task.RealVadRoot.traverse()):
                try:
                    self.write_vad_short(renderer, vad)
                except AttributeError:
                    pass

                try:
                    self.write_vad_control(renderer, vad)
                except AttributeError:
                    pass

                try:
                    self.write_vad_ext(renderer, vad)
                except AttributeError:
                    pass

                renderer.format("\n")

            self.session.report_progress("Pid %s: %s Vads" % (
                task.UniqueProcessId, count))

    def write_vad_short(self, renderer, vad):
        """Renders a text version of a Short Vad"""
        renderer.table_header([("VAD node @", "offset", ""),
                               ("address", "address", "[addrpad]"),
                               ("Start", "Start", "5"),
                               ("startaddr", "startaddr", "[addrpad]"),
                               ("End", "End", "3"),
                               ("endaddr", "endaddr", "[addrpad]"),
                               ("Tag", "Tag", "3"),
                               ("tagval", "tagval", ""),
                              ], suppress_headers=True)
        renderer.table_row("VAD node @",
                           vad.obj_offset,
                           "Start",
                           vad.Start,
                           "End",
                           vad.End,
                           "Tag",
                           vad.Tag)

        renderer.format("Flags: {0}\n", vad.u.VadFlags)

        # although the numeric value of Protection is printed above with
        # VadFlags, let's show the user a human-readable translation of the
        # protection
        renderer.format("Protection: {0}\n", vad.u.VadFlags.ProtectionEnum)

        # translate the vad type if its available (> XP)
        if vad.u.VadFlags.m("VadType"):
            renderer.format("Vad Type: {0}\n", vad.u.VadFlags.VadTypeEnum)

    def write_vad_control(self, renderer, vad):
        """Renders a text version of a (non-short) Vad's control information"""
        # even if the ControlArea is not NULL, it is only meaningful
        # for shared (non private) memory sections.
        if vad.u.VadFlags.PrivateMemory == 1:
            return

        control_area = vad.ControlArea
        if not control_area:
            return

        renderer.format("ControlArea @{0:08x} Segment {1:08x}\n",
                        control_area.dereference().obj_offset,
                        control_area.Segment)

        renderer.format("Dereference list: Flink {0:08x}, Blink {1:08x}\n",
                        control_area.DereferenceList.Flink,
                        control_area.DereferenceList.Blink)

        renderer.format(
            "NumberOfSectionReferences: {0:10} NumberOfPfnReferences:  "
            "{1:10}\n", control_area.NumberOfSectionReferences,
            control_area.NumberOfPfnReferences)

        renderer.format(
            "NumberOfMappedViews:       {0:10} NumberOfUserReferences: "
            "{1:10}\n", control_area.NumberOfMappedViews,
            control_area.NumberOfUserReferences)

        renderer.format(
            "WaitingForDeletion Event:  {0:08x}\n",
            control_area.WaitingForDeletion)

        renderer.format(
            "Control Flags: {0}\n", control_area.u.Flags)

        file_object = control_area.FilePointer
        if file_object and file_object != 0:
            renderer.format(
                "FileObject @{0:08x} FileBuffer @ {1:08x}          , "
                "Name: {2}\n", file_object.obj_offset,
                file_object.FileName.Buffer, file_object.FileName)

    def write_vad_ext(self, renderer, vad):
        """Renders a text version of a Long Vad"""
        if vad.obj_type != "_MMVAD_SHORT":
            renderer.format(
                "First prototype PTE: {0:08x} Last contiguous PTE: "
                "{1:08x}\n", vad.FirstPrototypePte[0],
                vad.LastContiguousPte)

            renderer.format("Flags2: {0}\n", vad.u2.VadFlags2)



class VADTree(VADInfo):
    """Walk the VAD tree and display in tree format"""

    __name = "vadtree"

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.format(u"Pid: {0:6}\n", task.UniqueProcessId)

            renderer.table_header([
                dict(name="", type="TreeNode", width=1),
                ("Start", "start", "[addrpad]"),
                ("-", "-", ""),
                ("End", "End", "[addrpad]")
                ], suppress_headers=True)

            for vad in task.RealVadRoot.traverse():
                level = vad.obj_context.get('depth', 0)
                renderer.table_row("", vad.Start, "->", vad.End, depth=level)


class VADWalk(VADInfo):
    """Walk the VAD tree"""

    __name = "vadwalk"

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.format(u"Pid: {0:6}\n", task.UniqueProcessId)
            renderer.table_header([("Address", "address", "[addrpad]"),
                                   ("Parent", "parent", "[addrpad]"),
                                   ("Left", "left", "[addrpad]"),
                                   ("Right", "right", "[addrpad]"),
                                   ("Start", "start", "[addrpad]"),
                                   ("End", "end", "[addrpad]"),
                                   ("Tag", "tag", "4"),
                                  ])
            for vad in task.RealVadRoot.traverse():
                # Ignore Vads with bad tags (which we explicitly include as
                # None)
                if vad:
                    renderer.table_row(
                        vad.obj_offset,
                        vad.obj_parent.obj_offset,
                        vad.LeftChild.v(),
                        vad.RightChild.v(),
                        vad.Start,
                        vad.End,
                        vad.Tag)

class VAD(common.WinProcessFilter):
    """Concise dump of the VAD.

    Similar to windbg's !vad.
    """

    __name = "vad"

    @classmethod
    def args(cls, parser):
        super(VAD, cls).args(parser)
        parser.add_argument(
            "--regex", default=None,
            help="A regular expression to filter VAD filenames.")

        parser.add_argument(
            "--offset", default=None, type="IntParser",
            help="Only print the vad corresponding to this offset.")

    def __init__(self, *args, **kwargs):
        self.regex = kwargs.pop("regex", None)
        self.offset = kwargs.pop("offset", None)
        self._cache = {}

        # Pass positional args to the WinProcessFilter constructor.
        super(VAD, self).__init__(*args, **kwargs)

    def find_file(self, addr):
        """Finds the file mapped at this address."""
        for task in self.filter_processes():
            yield self.find_file_in_task(addr, task)

    def find_file_in_task(self, addr, task):
        resolver = self.GetVadsForProcess(task)
        if resolver:
            start, end, data = resolver.get_containing_range(addr)
            return data

    def _make_cache(self, task):
        result = utils.RangedCollection()
        self.session.report_progress(
            " Enumerating VADs in %s (%s)", task.name, task.pid)

        for vad in task.RealVadRoot.traverse():
            result.insert(vad.Start, vad.End, (self._get_filename(vad), vad))

        return result

    def GetVadsForProcess(self, task):
        try:
            resolver = self._cache[task]
        except KeyError:
            # Break the recursion by placing a None for the resolver. The
            # self._make_cache() call will run the vad plugin which might
            # resolve a VAD PTE, calling this code. This means that we can only
            # use non-VAD PTEs to read the VAD itself.
            self._cache[task] = None
            resolver = self._cache[task] = self._make_cache(task)

        return resolver

    def _get_filename(self, vad):
        filename = ""
        try:
            file_obj = vad.ControlArea.FilePointer
            if file_obj:
                filename = file_obj.FileName or "Pagefile-backed section"
        except AttributeError:
            pass

        return unicode(filename)

    def render_vadroot(self, renderer, vad_root, task):
        renderer.table_header([('VAD', 'offset', '[addrpad]'),
                               ('lev', 'level', '>3'),
                               ('Start Addr', 'start_pfn', '[addrpad]'),
                               ('End Addr', 'end_pfn', '[addrpad]'),
                               dict(name='com', width=6, align="r"),
                               ('', 'type', '7'),
                               ('', 'executable', '6'),
                               ('Protect', 'protection', '!20'),
                               ('Filename', 'filename', '')])

        task_as = task.get_process_address_space()

        result = []
        for vad in vad_root.traverse():
            # Apply filters if needed.
            if self.regex and not re.search(
                    self.regex, self._get_filename(vad)):
                continue

            if (self.offset is not None and
                    not vad.Start <= self.offset <= vad.End):
                continue

            exe = ""
            if "EXECUTE" in str(vad.u.VadFlags.ProtectionEnum):
                exe = "Exe"

            result.append((
                vad, vad.obj_context.get('depth', 0),

                # The vad region itself exists in the process address space.
                self.profile.Pointer(value=vad.Start, vm=task_as),
                self.profile.Pointer(value=vad.End, vm=task_as),
                vad.CommitCharge if vad.CommitCharge < 0x7fffffff else -1,
                "Private" if vad.u.VadFlags.PrivateMemory > 0 else "Mapped",
                exe,
                vad.u.VadFlags.ProtectionEnum,
                self._get_filename(vad)))

        # Sort by start range.
        result.sort(key=lambda x: x[2].v())
        for row in result:
            renderer.table_row(*row)

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.format("Pid: {0} {1}\n", task.UniqueProcessId,
                            task.ImageFileName)
            renderer.RenderProgress("Pid: %s" % task.UniqueProcessId)
            self.render_vadroot(renderer, task.RealVadRoot, task)


class VADMap(pfn.VADMapMixin, common.WinProcessFilter):
    """Inspect each page in the VAD and report its status.

    This allows us to see the address translation status of each page in the
    VAD.
    """

    def _CreateMetadata(self, collection):
        metadata = {}
        for descriptor_cls, _, _ in collection.descriptors:
            # Any of the following kinds of PTE are considered prototype.
            if descriptor_cls in (pagefile.WindowsProtoTypePTEDescriptor,
                                  pagefile.VadPteDescriptor):
                metadata["ProtoType"] = True
                break

        for descriptor_cls, args, kwargs in reversed(collection.descriptors):
            if issubclass(descriptor_cls,
                          pagefile.WindowsSubsectionPTEDescriptor):
                metadata.update(descriptor_cls(
                    session=self.session, *args, **kwargs).metadata())

                return metadata

            elif issubclass(descriptor_cls, pagefile.WindowsPTEDescriptor):
                type = kwargs.get("pte_type")
                if type == "Proto":
                    metadata["ProtoType"] = True

                pte_value = kwargs.get("pte_value")
                if pte_value & 1:
                    metadata["type"] = "Valid"
                else:
                    metadata["type"] = "Transition"

                return metadata

            elif issubclass(descriptor_cls, pagefile.WindowsPagefileDescriptor):
                metadata["number"] = kwargs.get("pagefile_number", 0)
                metadata["offset"] = kwargs.get("address", 0)
                metadata["type"] = "Pagefile"

                return metadata

            elif issubclass(descriptor_cls, pagefile.DemandZeroDescriptor):
                metadata["type"] = "Demand Zero"
                return metadata

            elif issubclass(descriptor_cls, intel.PhysicalAddressDescriptor):
                metadata["offset"] = kwargs["address"]
                metadata.setdefault("type", "Valid")

            elif issubclass(descriptor_cls, intel.InvalidAddress):
                metadata["type"] = "Invalid"

        return metadata

    def _GenerateVADRuns(self, vad):
        address_space = self.session.GetParameter("default_address_space")

        offset = vad.Start
        end = vad.End

        while offset < end:
            if self.start <= offset <= self.end:
                yield offset, self._CreateMetadata(
                    address_space.describe_vtop(offset))

                self.session.report_progress("Inspecting 0x%08X", offset)

            offset += 0x1000

    def GeneratePageMetatadata(self, task):
        for module in self.session.address_resolver.modules():
            if isinstance(module, address_resolver.VadModule):
                # Skip guard regions for Wow64.
                if module.vad.CommitCharge >= 0x7fffffff:
                    continue

                for vaddr, metadata in self._GenerateVADRuns(module.vad):
                    yield vaddr, metadata


class VADDump(core.DirectoryDumperMixin, VAD):
    """Dumps out the vad sections to a file"""

    __name = "vaddump"

    @classmethod
    def args(cls, parser):
        super(VADDump, cls).args(parser)
        parser.add_argument("--max_size", type="IntParser",
                            default=100*1024*1024,
                            help="Maximum file size to dump.")

    def __init__(self, *args, **kwargs):
        self.max_size = kwargs.pop("max_size", 100*1024*1024)
        super(VADDump, self).__init__(*args, **kwargs)

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section("{0:6} ({1:2})".format(
                task.name, task.UniqueProcessId))

            renderer.table_header([
                ("Start", "start", "[addrpad]"),
                ("End", "end", "[addrpad]"),
                ("Length", "length", "[addr]"),
                ("Filename", "filename", "60s"),
                ("Comment", "comment", "")])

            # Get the task and all process specific information
            task_space = task.get_process_address_space()

            name = task.ImageFileName
            offset = task_space.vtop(task.obj_offset)
            if offset is None:
                renderer.format(
                    "Process does not have a valid address space.\n")
                continue

            with self.session.plugins.cc() as cc:
                cc.SwitchProcessContext(task)

                for vad in task.RealVadRoot.traverse():
                    # Find the start and end range
                    start = vad.Start
                    end = vad.End

                    filename = "{0}.{1:x}.{2:08x}-{3:08x}.dmp".format(
                        name, offset, start, end)

                    with renderer.open(directory=self.dump_dir,
                                       filename=filename,
                                       mode='wb') as fd:

                        if end - start > self.max_size:
                            renderer.table_row(start, end, end-start,
                                               "Skipped - Region too large")
                            continue

                        self.session.report_progress("Dumping %s" % filename)

                        self.CopyToFile(task_space, start, end + 1, fd)
                        renderer.table_row(
                            start, end, end-start, filename,
                            self._get_filename(vad))


class VadScanner(scan.BaseScanner):
    """A scanner over all memory regions of a process."""

    def __init__(self, task=None, process_profile=None, **kwargs):
        """Scan the process address space through the Vads.

        Args:
          task: The _EPROCESS object for this task.

          process_profile: The specialized profile for this process. In practice
            this is always different from task.obj_profile (which belongs to the
            kernel). If not provided we default to the kernel profile.
        """
        self.task = task
        super(VadScanner, self).__init__(
            profile=process_profile or task.obj_profile,
            address_space=task.get_process_address_space(),
            **kwargs)

    def scan(self, offset=0, maxlen=None):
        maxlen = maxlen or self.profile.get_constant("MaxPointer")

        for vad in self.task.RealVadRoot.traverse():
            # Only scan the VAD region.
            for match in super(VadScanner, self).scan(vad.Start, vad.Length):
                yield match



# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""The module implements the base class for address resolution."""

__author__ = "Michael Cohen <scudette@gmail.com>"

import re

from rekall import config
from rekall import obj
from rekall import utils


config.DeclareOption(
    "--name_resolution_strategies", default=["Module", "Symbol", "Export"],
    group="Interface", type="ChoiceArray",
    choices=["Module", "Symbol", "Export"])


class Module(object):
    """A range in the virtual address space which maps an executable.

    Each binary in the address space has its own profile, which knows about
    symbols within it. This simple class is just a container to facilitate
    access to the profile that represents this module.

    Within Rekall, each module has a name. Rekall uses a simple syntax to refer
    to an address in the address space by name (see below).
    """
    def __init__(self, name=None, start=None, end=None, profile=None,
                 session=None):
        self.name = name
        self.start = int(start)
        self.end = int(end)
        self.profile = profile
        self.session = session

    def __str__(self):
        return "%s: %s" % (self.__class__.__name__, self.name)


class AddressResolverMixin(object):

    """The basic building block for constructing an address resolver plugin.

    An address resolver maintains a collection of Modules and abstracts access
    to specific symbol names within the modules.

    Rekall uses a symbolic notation to refer to specific addresses within the
    address space. The address resolver is responsible for parsing this notation
    and resolving it to an actual address.

    Rules of symbol syntax
    ======================

    The address space is divided into "modules". A module has a name, a start
    address and an end address. Modules can also contain a profile which knows
    about symbols related to that module.

    1. Module reference: The start address of a module can be refered to by its
       name. e.g:  "nt", "ntdll", "tcpip".

    2. If a module contains a valid profile, the profile may also know about
       symbols within the module. We can refer to these
       symbols. e.g. "nt!MmGetIoSessionState"

    3. If an exact symbol is not found, it can be referred to with an offset
       from another symbol name. e.g. "nt!MmGetIoSessionState+5FE" (Note
       integers are given in hex).

    4. If the symbol is preceeded with a "*" - it means that the symbol is a
       pointer. The address will be read as a pointer and the symbol name will
       resolve to the address of the pointer's target.

    """

    # The name of the plugin.
    name = "address_resolver"

    # The format of a symbol name. Used by get_address_by_name().
    ADDRESS_NAME_REGEX = re.compile(
        r"(?P<deref>[*])?"              # Pointer dereference.

        r"((?P<address>0x[0-9A-Fa-f]+)|" # Alternative - Either an address, or,

        r"(?P<module>[A-Za-z_0-9\.\\]+)" # Module name - can include extension
                                         # (.exe, .sys)

        r"!?"                           # ! separates module name from symbol
                                        # name.

        r"(?P<symbol>[^ +-]+)?"         # Symbol name.
        r")"                            # End alternative.

        r"(?P<op> *[+-] *)?"            # Possible arithmetic operator.
        r"(?P<offset>[0-9a-fA-Fx]+)?")  # Possible hex offset.

    def __init__(self, **kwargs):
        super(AddressResolverMixin, self).__init__(**kwargs)
        self.reset()

    def reset(self):
        # A ranged collection of Module() objects.
        self._address_ranges = utils.RangedCollection()

        # A lookup between module names and the Module object itself.
        self._modules_by_name = {}

        self._initialized = False

    def NormalizeModuleName(self, module_name):
        if module_name is not None:
            module_name = unicode(module_name)
            module_name = re.split(r"[/\\]", module_name)[-1]

            return module_name.lower()

    def _EnsureInitialized(self):
        """Initialize this address resolver."""

    def AddModule(self, module):
        self._address_ranges.insert(module.start, module.end, module)
        if module.name:
            self._modules_by_name[module.name] = module

    def _ParseAddress(self, name):
        """Parses the symbol from Rekall symbolic notation.

        Raises:
          TypeError if the expression has a syntax error.

        Returns:
          a dict containing the different components of the expression.
        """
        m = self.ADDRESS_NAME_REGEX.match(name)
        if m:
            capture = m.groupdict()
            if not capture.get("address"):
                module = capture.get("module")
                if not module:
                    raise TypeError("Module name not specified.")

                capture["module"] = self.NormalizeModuleName(module)

            if capture["op"] and not (capture["symbol"] or
                                      capture["address"] or
                                      capture["module"]):
                raise TypeError("Operator %s must have an operand." %
                                capture["op"])

            if capture["op"] and not (capture["symbol"] or capture["address"] or
                                      capture["module"]):
                raise TypeError(
                    "Operator %s must operate on a symbol or address." %
                    capture["op"])

            return capture

        raise TypeError("Unable to parse %r as a symbol name" % name)

    def modules(self):
        self._EnsureInitialized()
        for _, _, module in self._address_ranges:
            yield module

    def GetContainingModule(self, address):
        """Finds the module containing the specified address.

        Returns:
          A Module() instance.
        """
        self._EnsureInitialized()
        address = obj.Pointer.integer_to_address(address)

        _, _, module = self._address_ranges.get_containing_range(address)
        return module

    def GetModuleByName(self, name):
        self._EnsureInitialized()
        return self._modules_by_name.get(self.NormalizeModuleName(name))

    def get_constant_object(self, name, target, **kwargs):
        """Instantiate the named constant with these args.

        This method is the main entry point for instantiating constants. It is
        preferred than calling the profile's method of the same name directly
        since it will be responsible with loading the right profile.
        """
        self._EnsureInitialized()

        # Parse the name
        components = self._ParseAddress(name)
        if not components["symbol"]:
            raise ValueError("No symbol name specified.")

        module = self._modules_by_name.get(components["module"])
        if module is not None:
            # Just delegate to the module's profile.
            if module.profile:
                return module.profile.get_constant_object(
                    components["symbol"], target=target, **kwargs)

        return obj.NoneObject("Profile for name %s unknown." % name, log=True)

    def get_address_by_name(self, name):
        """Convert the symbol annotated by name to an address."""
        self._EnsureInitialized()

        try:
            return int(name)
        except (ValueError, TypeError):
            pass

        if not isinstance(name, basestring):
            raise TypeError("Name should be a string.")

        module = None
        components = self._ParseAddress(name)
        module_name = self.NormalizeModuleName(components["module"])
        address = components["address"]
        if address is not None:
            address = int(address, 0)
        # User did not specify an address
        else:
            module = self._modules_by_name.get(module_name)
            if not module:
                return obj.NoneObject(
                    "No module %s found" % module_name, log=True)

            # Found the module we use its base address
            address = module.start

        # Search for a symbol in the module.
        symbol = components["symbol"]
        if symbol:
            # Get the profile for this module.
            if module.profile:
                address = module.profile.get_constant(symbol, is_address=True)

            else:
                return obj.NoneObject("No profile found for module", log=True)

        # Support basic offset operations (+/-).
        op = components["op"]
        if op:
            op = op.strip()
            # Parse the offset as hex or decimal.
            offset = int(components["offset"], 0)
            if op == "+":
                address += offset
            elif op == "-":
                address -= offset
            else:
                raise TypeError("Operator '%s' not supported" % op)

        # If the symbol was a dereference, we need to read the address from
        # this offset.
        if components.get("deref"):
            try:
                address = module.profile.Pointer(address).v()
            except AttributeError:
                address = self.session.profile.Pointer(address).v()

        return address

    def format_address(self, address, max_distance=0x1000000):
        """Format the address as a symbol name.

        This means to try and find the containing module, the symbol within the
        module or possibly an offset from a known symbol. e.g.

        nt!PspCidTable
        nt!PspCidTable + 0x10
        nt + 0x234

        Returns a list of symbol names for the address. The list is empty if the
        address is not in a containing module if the nearest known symbol is
        farther than max_distance away.
        """
        self._EnsureInitialized()

        _, symbols = self.get_nearest_constant_by_address(
            address, max_distance=max_distance)

        return sorted(symbols)

    def get_nearest_constant_by_address(self, address, max_distance=0x1000000):
        """Searches for a known symbol at an address lower than this.

        Returns a tuple (nearest_offset, list of symbol names).
        """
        self._EnsureInitialized()

        address = obj.Pointer.integer_to_address(address)
        symbols = []
        module = self.GetContainingModule(address)
        if not module or not module.name:
            return (-1, [])

        if module.profile != None:
            offset, symbols = module.profile.get_nearest_constant_by_address(
                address)

        # Symbols not found at all
        if not symbols:
            if address - module.start > max_distance:
                return (-1, [])

            if address == module.start:
                return (module.start, [module.name])

            return (module.start, [
                "%s+%#x" % (module.name, address - module.start)])

        if address - offset > max_distance:
            return (-1, [])

        # Exact symbols found.
        if offset == address:
            return (offset, ["%s!%s" % (module.name, x) for x in symbols])

        return (offset, ["%s!%s+%#x" % (module.name, x, address - offset)
                         for x in symbols])

    def search_symbol(self, pattern):
        """Searches symbols for the pattern.

        pattern may contain wild cards (*). Note that currently a module name is
        required. Example pattern:

        nt!Ps*
        """
        self._EnsureInitialized()
        result = []

        components = self._ParseAddress(pattern)
        module_name = self.NormalizeModuleName(components["module"])
        if module_name == None:
            raise RuntimeError(
                "Module name must be specified for symbol search.")

        module = self._modules_by_name.get(module_name)
        if module:
            # Match all symbols.
            symbol_regex = re.compile(components["symbol"].replace("*", ".*"))
            if module.profile:
                for constant in module.profile.constants:
                    if symbol_regex.match(constant):
                        result.append("%s!%s" % (module_name, constant))

        return result



import mock
from rekall import testlib
from rekall.plugins.tools import dynamic_profiles


class MockAddressResolver(object):
    def __init__(self, name_map):
        self.name_map = name_map

    def format_address(self, address):
        return self.name_map.get(address, "")


class TestDynamicProfile(testlib.RekallBaseUnitTestCase):
    """Tests the dynamic profile mechanism."""

    TEST_CASES = [
        dict(
            expected={"$out": 0x20},
            mode="AMD64",
            # Taken from Windows 7 x64
            offset=0xf800029eb5d0,
            data=('48895c240848896c24104889742418574883ec2033ff418bf08bea488bd'
                  '948393975218d572833c941b848546162e80d4602004889034885c07504'
                  '32c0eb49bf01000000488b1b33d2448d4228488bcbe86b27efff83630c0'
                  '00bfe893bc7430880000000c743107f000000896b04e80583e9ff4885c0'
                  '750a488bcbe8f0feffffebb948894320b001488b5c2430'),
            example="""
0xf800029eb63e           0x6e e80583e9ff           call 0xf80002883948                      nt!RtlpAllocateSecondLevelDir
0xf800029eb643           0x73 4885c0               test rax, rax
0xf800029eb646           0x76 750a                 jne 0xf800029eb652                       nt!RtlCreateHashTable+0x82
0xf800029eb648           0x78 488bcb               mov rcx, rbx
0xf800029eb64b           0x7b e8f0feffff           call 0xf800029eb540                      nt!RtlDeleteHashTable
0xf800029eb650           0x80 ebb9                 jmp 0xf800029eb60b                       nt!RtlCreateHashTable+0x3b
0xf800029eb652           0x82 48894320             mov qword ptr [rbx + 0x20], rax
""",
            rules=[
                {'mnemonic': 'CALL',
                 'comment': 'nt!RtlpAllocateSecondLevelDir'},

                {'mnemonic': 'MOV',
                 'operands': [{'disp': "$out", 'base': '$rbx'},
                              {'type': 'REG', 'reg': 'RAX'}]},
            ],
            # Used to pre-seed the address resolver with symbol names for
            # testing.
            name_map={
                0xf80002883948: ["nt!RtlpAllocateSecondLevelDir"],
            },
        ),


        # Example from MiSessionInsertImage()
        # http://gate.upm.ro/os/LABs/Windows_OS_Internals_Curriculum_Resource_Kit-ACADEMIC/WindowsResearchKernel-WRK/WRK-v1.2/base/ntos/mm/sessload.c
        dict(
            # Taken from Windows 8 x64 dis "nt!MiSessionInsertImage"
            offset=0xf801ea55f680,
            data=('48895c240848896c2410488974241857415641574883ec20498bf0488bea'
                  '488bf941be5000000041b84d6d4869b900020000418bd6e856091200488b'
                  'd84885c00f84fee60900458bc633d2488bc8e89d03f3ffc7433001000000'
                  '4883cf0348897b20654c8b342588010000498b86b8000000488b88f00300'
                  '008b41084c8db9f80b0000488d7968498bd7498bce48896b38894334e8ef'
                  '16f7ff4c8b1f4c3bdf'),
            rules=[
                {'mnemonic': 'MOV', 'operands': [
                    {'type': 'REG', 'reg': '$RDI'},
                    {'type': 'REG', 'reg': 'RCX'}]},

                {'mnemonic': 'CALL',
                 'comment': 'nt!ExAllocatePoolWithTag'},

                {'mnemonic': 'MOV', 'operands': [
                    {'type': 'REG', 'reg': '$RBX'},
                    {'type': 'REG', 'reg': 'RAX'}]},

                # RtlZeroMemory (NewImage, sizeof(IMAGE_ENTRY_IN_SESSION));
                {'mnemonic': 'CALL', 'comment': 'nt!memset'},

                # NewImage->ImageCountInThisSession = 1;
                {'mnemonic': 'MOV', 'operands': [
                    {'disp': "$ImageCountInThisSession",
                     'base': '$RBX', 'type': 'MEM'},
                    {'address': 1, 'type': 'IMM'}]},

                # NewImage->Address = BaseAddress;
                {'mnemonic': 'MOV', 'operands': [
                    {'disp': "$Address",
                     'base': '$RBX', 'type': 'MEM'},
                    {'type': 'REG', 'reg': '$RDI'}]},
            ],
            name_map={
                0xf801ea680010: ["nt!ExAllocatePoolWithTag"],
                0xf801ea48fa70: ["nt!memset"],
            },
            expected={"$Address": 0x20, "$ImageCountInThisSession": 0x30},
        ),

    ]

    def testDynamicProfile(self):
        for case in self.TEST_CASES:
            self.session = mock.Mock(
                wraps=self.MakeUserSession(),
                address_resolver=MockAddressResolver(
                    case.get("name_map", {}))
            )

            matcher = dynamic_profiles.DisassembleMatcher(
                mode=case.get("mode", "AMD64"),
                rules=case["rules"],
                session=self.session)

            match = matcher.Match(offset=case.get("offset", 0),
                                  data=case["data"].decode("hex"))

            for k, v in case["expected"].iteritems():
                self.assertEqual(match[k], v)



# Rekall Memory Forensics
# Copyright (C) 2007-2013 Volatility Foundation
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This file is part of Rekall Memory Forensics.
#
# Rekall Memory Forensics is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General Public
# License.
#
# Rekall Memory Forensics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Rekall Memory Forensics.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization:
"""

from rekall.plugins.linux import common
from rekall.plugins.tools import dynamic_profiles


class CheckSyscall(common.LinuxPlugin):
    """Checks if the system call table has been altered."""

    __name = "check_syscall"

    def Find_sys_call_tables(self):
        """Calculates the size of the syscall table.

        Here we need the symbol __NR_syscall_max. We derive it from
        disassembling the following system calls:

        - system_call_fastpath function:

        http://lxr.linux.no/linux+v3.12/arch/x86/kernel/entry_64.S#L620
        system_call_fastpath:
        #if __SYSCALL_MASK == ~0
                cmpq $__NR_syscall_max,%rax
        #else
                andl $__SYSCALL_MASK,%eax
                cmpl $__NR_syscall_max,%eax
        #endif

        - ret_from_sys_call function (with a small rewind):
        http://lxr.linux.no/linux+v2.6.26/arch/x86/kernel/entry_64.S#L249

        249        cmpq $__NR_syscall_max,%rax
        250        ja badsys
        251        movq %r10,%rcx
        252        call *sys_call_table(,%rax,8)  # XXX:    rip relative
        253        movq %rax,RAX-ARGOFFSET(%rsp)
        254 /*
        255  * Syscall return path ending with SYSRET (fast path)
        256  * Has incomplete stack frame and undefined top of stack.
        257  */
        258 ret_from_sys_call:
        259        movl $_TIF_ALLWORK_MASK,%edi
        260        /* edi: flagmask */


        - sysenter_do_call
           Linux> dis "linux!sysenter_do_call"
           Address    Rel           Op Codes           Instruction    Comment
           ------- ---------- -------------------- ------------------ -------
           ------ linux!sysenter_do_call ------: 0xc12c834d
           0xc12c834d        0x0 3d5d010000           CMP EAX, 0x15d
           0xc12c8352        0x5 0f8397baffff         JAE 0xc12c3def  linux!syscall_badsys

        """
        rules = [
            # Look for a comparison of the register (EAX) with a fixed value.
            {'mnemonic': 'CMP', 'operands': [
                {'type': 'REG'}, {'type': 'IMM', 'target': "$value"}]},

            # Immediately followed by a branch to linux!badsys,
            # linux!ia32_badsys etc.
            {'comment': '~.+badsys'}
        ]
        func = None
        tables = set()
        for func_name, table_name in [
                # http://lxr.free-electrons.com/source/arch/x86_64/kernel/entry.S?v=2.4.37
                ("system_call", "sys_call_table"),
                # http://lxr.free-electrons.com/source/arch/x86/kernel/entry_64.S?v=3.16
                ("system_call_fastpath", "sys_call_table"),


                # http://lxr.free-electrons.com/source/arch/x86/ia32/ia32entry.S?v=3.14
                ("ia32_sysenter_target", "ia32_sys_call_table"),
                ("sysenter_auditsys", "ia32_sys_call_table"),

                # http://lxr.free-electrons.com/source/arch/x86/kernel/entry_32.S?v=3.3
                ("sysenter_do_call", "sys_call_table")]:

            if table_name in tables:
                continue

            # This table does not exist in this profile dont bother looking for
            # its size.
            if self.profile.get_constant(table_name) == None:
                continue

            func = self.profile.get_constant_object(
                func_name, target="Function")
            if func == None:
                continue

            matcher = dynamic_profiles.DisassembleMatcher(
                name="sys_call_table_size",
                mode=func.mode, rules=rules, session=self.session)

            result = matcher.MatchFunction(func)
            if result:
                tables.add(table_name)
                yield table_name, result["$value"] + 1

        # Fallback. Note this underestimates the size quite a bit.
        if func == None:
            table_size = len([x for x in self.profile.constants
                              if x.startswith("__syscall_meta__")]) or 0x300
            yield "ia32_sys_call_table", table_size
            yield "sys_call_table", table_size

    def CheckSyscallTables(self):
        """
        This works by walking the system call table
        and verifies that each is a symbol in the kernel
        """
        for table_name, table_size in  self.Find_sys_call_tables():
            # The syscall table is simply an array of pointers to functions.
            table = self.profile.get_constant_object(
                table_name,
                target="Array",
                target_args=dict(
                    count=table_size,
                    target="Pointer",
                    target_args=dict(
                        target="Function"
                        )
                    )
                )

            resolver = self.session.address_resolver
            for i, entry in enumerate(table):
                yield (table_name, i, entry,
                       resolver.format_address(entry.deref()))

    def render(self, renderer):
        renderer.table_header([
            ("Table Name", "table", "20"),
            ("Index", "index", "[addr]"),
            ("Address", "address", "[addrpad]"),
            ("Symbol", "symbol", "<130")])

        for table_name, i, call_addr, sym_name in self.CheckSyscallTables():
            renderer.table_row(table_name, i, call_addr, sym_name or "Unknown",
                               highlight=None if sym_name else "important")



# Rekall Memory Forensics
#
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
Darwin Session collectors and plugins.
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import plugin

from rekall.plugins.darwin import common


class DarwinSessions(common.AbstractDarwinProducer):
    """Finds sessions by walking their global hashtable."""

    name = "sessions"
    type_name = "session"

    def collect(self):
        session_hash_table_size = self.profile.get_constant_object(
            "_sesshash", "unsigned long")

        # The hashtable is an array to session list heads.
        session_hash_table = self.profile.get_constant_object(
            "_sesshashtbl",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    target="sesshashhead",
                    count=session_hash_table_size.v())))

        # We iterate over the table and then over each list.
        for sesshashhead in session_hash_table:
            for session in sesshashhead.lh_first.walk_list("s_hash.le_next"):
                yield [session]


class DarwinTerminals(common.AbstractDarwinTypedCommand):
    """Lists open ttys."""

    name = "terminals"

    table_header = plugin.PluginHeader(
        dict(name="Session", type="session", cname="session",
             columns=[dict(name="Session ID", cname="s_sid")]),
        dict(name="Terminal", type="tty", cname="tty")
    )

    def collect(self):
        for session in self.session.plugins.sessions().produce():
            if session.s_ttyp:
                yield [session, session.s_ttyp]



# Rekall Memory Forensics
#
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@google.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements dynamic profiles.

A Dynamic profile is a way of discovering certain parameters via running a
matching signature.
"""
from rekall import obj
from rekall.plugins.tools import disassembler


class DisassembleMatcher(object):
    """A matching engine for disassembler rules.

    This matcher searcher for a sequence of rules in a disassmbly and tries to
    match a certain rule pattern to the assembly. Ultimately if the rules match,
    the rules may extract certain parameters from the patter.
    """

    def __init__(self, name="", mode="AMD64", rules=None, session=None,
                 max_separation=10):
        self.mode = mode
        self.name = name
        self.rules = rules
        self.session = session
        self.max_separation = max_separation
        self.dis = disassembler.Capstone(self.mode, session=self.session)

    def _CheckCaptureVariables(self, vector, contexts):
        """Checks that capture variables are consistent in the vector.

        The vector is a list of disassembly lines which match the rules, e.g.

        [16, 60, 61]

        The context is the capture variables from these rules. In order
        to be valid, the capture variables must all be consistent. For
        example the following is not consistent (since var1 is RAX in
        the first rule and RCX in the second rule):

        contexts[16]
        {'var1': u'RAX'}

        contexts[60]
        {'var1': u'RCX', 'out': u'0x88'}

        contexts[61]
        {}
        """
        result = {}
        for rule_number, item in enumerate(vector):
            rule_context = contexts[rule_number]
            # The capture variables in this rule only.
            rule_capture_vars_values = {}

            for k, v in rule_context[item].iteritems():
                var_name = k.rsplit("_", 1)[0]

                # Only consider variables (start with $).
                if not var_name.startswith("$"):
                    continue

                # If this var is previously known, this match must be the same
                # as previously found.
                if var_name in result and v != result[var_name]:
                    return

                # If this capture variable's value is the same as another
                # capture variable's value in the same rule, exclude the
                # match. This means that an expression like:
                #
                #     MOV $var2, [$var1+$out]
                #
                # Necessarily implies that $var1 and $var2 must be different
                # registers.
                if (v in rule_capture_vars_values and
                        rule_capture_vars_values[v] != var_name):
                    return

                result[var_name] = v
                rule_capture_vars_values[v] = var_name

        return result

    def _FindRuleIndex(self, instruction):
        """Generate all rules that match the current instruction."""
        for i, rule in enumerate(self.rules):
            context = dict(
                instruction=instruction.text, offset=instruction.address)
            if instruction.match_rule(rule, context):
                yield i, context

    def GenerateVector(self, hits, vector, level):
        """Generate possible hit vectors which match the rules."""
        for item in hits.get(level, []):
            if vector:
                if item < vector[-1]:
                    continue

                if item > self.max_separation + vector[-1]:
                    break

            new_vector = vector + [item]

            if level + 1 == len(hits):
                yield new_vector

            elif level + 1 < len(hits):
                for result in self.GenerateVector(hits, new_vector, level+1):
                    yield result

    def _GetMatch(self, hits, contexts):
        """Find the first vector that matches all the criteria."""
        for vector in self.GenerateVector(hits, [], 0):
            context = self._CheckCaptureVariables(vector, contexts)
            if not context:
                continue

            return (vector, context)

        return [], {}

    def MatchFunction(self, func, length=1000):
        return self.Match(
            func.obj_offset, func.obj_vm.read(func.obj_offset, length))

    def Match(self, offset=0, data=""):
        hits = {}
        contexts = {}

        for hit, instruction in enumerate(self.dis.disassemble(data, offset)):
            for rule_idx, context in self._FindRuleIndex(instruction):
                hits.setdefault(rule_idx, []).append(hit)
                contexts.setdefault(rule_idx, {})[hit] = context

        # All the hits must match
        if len(hits) < len(self.rules):
            self.session.logging.error("Failed to find match for %s", self.name)

            # Add some debugging messages here to make diagnosing errors easier.
            for i, rule in enumerate(self.rules):
                if i not in hits:
                    self.session.logging.debug("Unable to match rule: %s", rule)

            return obj.NoneObject()

        vector, context = self._GetMatch(hits, contexts)

        if len(vector) < len(self.rules):
            self.session.logging.error(
                "Failed to find match for %s - Only matched %s/%s rules.",
                self.name, len(vector), len(self.rules))
            return obj.NoneObject()

        self.session.logging.debug("Found match for %s", self.name)

        result = {}
        for i, hit in enumerate(vector):
            hit_data = contexts[i][hit]
            result.update(hit_data)
            self.session.logging.debug(
                "%#x %s", hit_data["offset"], hit_data["instruction"])

        return result


class DisassembleConstantMatcher(object):
    """Search for the value of global constants using disassembly."""

    def __init__(self, session, profile, name, args):
        self.session = session
        self.profile = profile
        self.args = args
        self.name = name
        # Start address to disassemble - can be an exported function name.
        self.start_address = args["start"]

        # Disassemble capture rules.
        self.rules = args["rules"]

    def __call__(self):
        resolver = self.session.address_resolver
        func = self.session.profile.Function(resolver.get_address_by_name(
            self.start_address))

        matcher = DisassembleMatcher(
            mode=func.mode, rules=self.rules, name=self.name,
            session=self.session)

        result = matcher.MatchFunction(func)
        if result and "$out" in result:
            return result["$out"]


class FirstOf(object):
    """Try a list of callables until one works."""
    def __init__(self, list_of_callables, **kwargs):
        self.list_of_callables = list_of_callables
        self.kwargs = kwargs

    def __call__(self, *args):
        for func in self.list_of_callables:
            result = func(*args, **self.kwargs)
            if result != None:
                return result


class DynamicConstantProfileLoader(obj.ProfileSectionLoader):
    """Produce a callable for a constant."""
    name = "$DYNAMIC_CONSTANTS"

    def LoadIntoProfile(self, session, profile, constants):
        """Parse the constants detectors and make callables."""
        for constant_name, rules in constants.items():
            detectors = []

            # Each constant can have several different detectors.
            for rule in rules:
                detector_name = rule["type"]
                detector_arg = rule["args"]

                # We only support one type of detector right now.
                if detector_name != "DisassembleConstantMatcher":
                    session.logging.error(
                        "Unimplemented detector %s", detector_name)
                    continue

                detectors.append(
                    DisassembleConstantMatcher(
                        session, profile, constant_name, detector_arg))

            profile.add_constants({constant_name: FirstOf(detectors)},
                                  constants_are_absolute=True)

        return profile


class DisassembleStructMatcher(DisassembleConstantMatcher):
    """Match a struct based on rules."""

    def __call__(self, struct, member=None):
        resolver = struct.obj_session.address_resolver
        func = struct.obj_profile.Function(resolver.get_address_by_name(
            self.start_address))

        matcher = DisassembleMatcher(
            mode=func.mode, rules=self.rules, name=self.name,
            max_separation=self.args.get("max_separation", 10),
            session=struct.obj_session)

        struct.obj_session.logging.info(
            "DisassembleStructMatcher: %s %s", self.name,
            self.args.get("comment", ""))
        result = matcher.MatchFunction(func)
        if result:
            # Match succeeded - create a new overlay for the Struct.
            overlay = {self.name: [None, {}]}
            fields = overlay[self.name][1]
            for field, field_args in self.args["fields"].iteritems():
                fields[field] = [result["$" + field], field_args]

            # This should never happen?
            if member not in fields:
                return

            # We calculated the types, now we add them to the profile so the
            # next time a struct is instantiated it will be properly
            # initialized.
            struct.obj_profile.add_types(overlay)

            # Now take care of the current struct which has already been
            # initialized.
            struct.members.update(struct.obj_profile.Object(self.name).members)

            # Return the member from the current struct.
            return struct.m(member)


class DynamicStructProfileLoader(obj.ProfileSectionLoader):
    """Produce a callable for a constant."""
    name = "$DYNAMIC_STRUCTS"

    def LoadIntoProfile(self, session, profile, data):
        """Parse the constants detectors and make callables."""
        overlay = {}
        for struct_name, signatures in data.items():
            detectors = {}

            # Each field can have several different detectors.
            for rule in signatures:
                detector_name = rule["type"]
                detector_arg = rule["args"]

                # We only support one type of detector right now.
                if detector_name != "DisassembleStructMatcher":
                    session.logging.error(
                        "Unimplemented detector %s", detector_name)
                    continue

                detector = DisassembleStructMatcher(
                    None, None, struct_name, detector_arg)

                # Add the detector to each field. The initial detector is a
                # pass-through which returns the normal member if one is defined
                # in the conventional way. If None is defined, we launch our
                # dynamic detector - which will store the conventional member
                # definitions as a cache.
                def PassThrough(struct, member=None):
                    return struct.m(member)

                for field in detector_arg["fields"]:
                    detectors.setdefault(field, [PassThrough]).append(detector)

            # Install an overlay with the chain of detectors.
            overlay[struct_name] = [None, {}]
            for field in detectors:
                overlay[struct_name][1][field] = FirstOf(
                    detectors[field], member=field)

        profile.add_overlay(overlay)
        return profile
